"""This parses a '.in' file from xwaves.   The '.in' files
show segmentation of an utterance.    segmentfile.parse(s)
takes a list of lines from such a file, and returns
three things:
* the title of the file (typically the utterance)
* a list of groups (a group is typically a word)
* a list of segments.

Groups include a list of the segments of which they are made,
along with a name and an index.
Segments contain a phoneme, a start/end time,
an index, and the group to which they belong.
"""

def _indented(s):
	return len(s)>0 and (s[0]=='\t' or s[0]==' ')

# sample .in file:
# ** o cen s 
# o	
#            o          1.620417
# cen	
#          Ccl          1.679583
#            C           1.76925
#            ^         1.8527499
#            N          1.941583
# s	
#            s         2.0380001

def _title(s):
	i = 0
	while s[i] == '*' :
		i = i + 1
	return s[i:].lstrip()

class segment:
	def __init__(self, st, en, phn, si):
		self.start = st
		self.end = en
		self.phn = phn
		self.segidx = si
		self.group = None

	def setgroup(self, g):
		self.group = g

	def __str__(self):
		return "<segment [%d]/%s[%d] %g-%g>" % (
					self.group.groupidx,
					self.phn, self.segidx,
					self.start, self.end )

	def __repr__(self):
		return self.__str__()



class group:
	def __init__(self, gi, gn):
		self.s = []
		self.groupidx = gi
		self.groupname = gn

	def add(self, seg):
		self.s.append(seg)

	def __str__(self):
		tmp = ', '.join(map( lambda x: x.__str__(), self.s))
		return "<group %s[%d] %s >" % (self.groupname, self.groupidx, tmp)

	def __repr__(self):
		return self.__str__()


def parse(l):
	"""Parses a list of lines from an ESPS Xmark file (.in),
	and returns a tuple (title, list of groups, list of segments)."""
	pseg = []
	pg = []
	title = None
	last_t = None
	groupidx = 0
	curgroup = group(groupidx, '')
	segidx = -1
	for line in l :
		line = line.rstrip()
		if not _indented(line):
			if title is None and len(line)>0 and line[0]=='*':
				title = _title(line)
			elif len(line)>0:
				curgroup = group(groupidx, line)
				pg.append(curgroup)
				groupidx += 1
		else:	# indented
			ss = line.lstrip().split()
			if(len(ss) != 2) :
				continue
			t = float(ss[1])
			if last_t is not None:
				seg = segment(last_t, t, ss[0], segidx)
				seg.setgroup(curgroup)
				curgroup.add(seg)
				pseg.append(seg)

			last_t = t
			segidx += 1
	return (title, pg, pseg)


def read(f):
	"""Read a ESPS Xmark (.in) file, and return information.  See parse()."""
	ii = open(f, "rb")
	o = parse(ii.readlines())
	ii.close()
	return o



def test():
	teststring = [	'** o cen s',
			'o',
			'           o          1.620417',
			'cen',
			'         Ccl          1.679583',
			'           C           1.76925',
			'           ^         1.8527499',
			'           N          1.941583',
			's	',
			'           s         2.0380001',
			'' ]
	title, grp, seg = parse(teststring)
	assert title == 'o cen s'
	assert seg[0].phn == 'Ccl'
	assert seg[0].start == 1.620417
	assert seg[0].end == 1.679583
	assert seg[0].group.groupidx == 1
	assert seg[0].segidx == 0
	assert grp[0].groupidx == 0
	assert grp[0].groupname == 'o'
	for t in grp:
		for q in t.s:
			assert q.group == t
	assert len(seg) == 5
	assert len(grp) == 3

if __name__ == '__main__':
	test()



#!/usr/bin/env python

"""convexhull.py

Calculate the convex hull of a set of n 2D-points in O(n log n) time.  
Taken from Berg et al., Computational Geometry, Springer-Verlag, 1997.
Prints output as EPS file.

When run from the command line it generates a random set of points
inside a square of given length and finds the convex hull for those,
printing the result as an EPS file.

Usage: convexhull.py <numPoints> <squareLength> <outFile>

Dinu C. Gherman

Small Bug: Only works with a list of UNIQUE points, Evan Jones, 2005/05/18
If the list of points passed to this function is not unique, it will raise an assertion.
To fix this, remove these lines from the beginning of the convexHull function:

Taken from http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/66527
and modified to work with complex numbers.

"""


import sys, random

class DuplicatePoints(ValueError):
	def __init__(self, s):
		ValueError.__init__(self, s)


######################################################################
# Helpers
######################################################################

def _myDet(p, q, r):
    """Calc. determinant of a special matrix with three 2D points.

    The sign, "-" or "+", determines the side, right or left,
    respectivly, on which the point r lies, when measured against
    a directed vector from p to q.
    """

    # We use Sarrus' Rule to calculate the determinant.
    # (could also use the Numeric package...)
    # sum1 = q[0]*r[1] + p[0]*q[1] + r[0]*p[1]
    # sum1 = q.real*r.imag + p.real*q.imag + r.real*p.imag
    # sum2 = q[0]*p[1] + r[0]*q[1] + p[0]*r[1]
    # sum2 = q.real*p.imag + r.real*q.imag + p.real*r.imag
    sum = q.real*(r.imag-p.imag) + p.real*(q.imag-r.imag) + r.real*(p.imag-q.imag)

    # return sum1 - sum2
    return sum


def _isRightTurn((p, q, r)):
    "Do the vectors pq:qr form a right turn, or not?"
    if p==q or p==r or q==r:
	if p==q:
		dup = p
	elif p==r:
		dup = p
	elif q==r:
		dup = q
    	raise DuplicatePoints, 'Two points are identical at %s' % str(dup)
    return _myDet(p, q, r) < 0


def _isPointInPolygon(r, P):
    "Is point r inside a given polygon P?"

    # We assume the polygon is a list of points, listed clockwise!
    for i in xrange(len(P[:-1])):
        p, q = P[i], P[i+1]
        if not _isRightTurn((p, q, r)):
            return False # Out!        

    return True # It's within!


def _makeRandomData(numPoints=30, sqrLength=100, addCornerPoints=0):
    "Generate a list of random points within a square."
    
    # Fill a square with random points.
    mn, mx = 0, sqrLength
    P = []
    for i in xrange(numPoints):
	rand = random.randint
	y = rand(mn+1, mx-1)
	x = rand(mn+1, mx-1)
	P.append(complex(x, y))

    # Add some "outmost" corner points.
    if addCornerPoints != 0:
	P = P + [complex(min, min), complex(max, max), complex(min, max), complex(max, min)]

    return P


######################################################################
# Output
######################################################################

epsHeader = """%%!PS-Adobe-2.0 EPSF-2.0
%%%%BoundingBox: %d %d %d %d

/r 2 def                %% radius

/circle                 %% circle, x, y, r --> -
{
    0 360 arc           %% draw circle
} def

/cross                  %% cross, x, y --> -
{
    0 360 arc           %% draw cross hair
} def

1 setlinewidth          %% thin line
newpath                 %% open page
0 setgray               %% black color

"""

def saveAsEps(P, H, boxSize, path):
    "Save some points and their convex hull into an EPS file."
    
    # Save header.
    f = open(path, 'w')
    f.write(epsHeader % (0, 0, boxSize, boxSize))

    format = "%3d %3d"

    # Save the convex hull as a connected path.
    if H:
        f.write("%s moveto\n" % format % (H[0].real, H[0].imag))
        for p in H:
            f.write("%s lineto\n" % format % (p.real, p.imag))
        f.write("%s lineto\n" % format % (H[0].real, H[0].imag))
        f.write("stroke\n\n")

    # Save the whole list of points as individual dots.
    for p in P:
        f.write("%s r circle\n" % format % (p.real, p.imag))
        f.write("stroke\n")
            
    # Save footer.
    f.write("\nshowpage\n")


######################################################################
# Public interface
######################################################################

def convexHull(P):
    """Calculate the convex hull of a set of complex points.
    If the hull has a duplicate point, an exception will be raised.
    It is up to the application not to provide duplicates.
    """

    # Remove any duplicates
    points = list(set(P))
    points.sort(lambda a, b: 2*cmp(a.real, b.real)+cmp(a.imag, b.imag))

    # Build upper half of the hull.
    upper = [points[0], points[1]]
    for p in points[2:]:
	upper.append(p)
	while len(upper)>2 and not _isRightTurn(upper[-3:]):
	    del upper[-2]
    while len(upper)>2 and not _isRightTurn(upper[-2:]+upper[:1]):
    	del upper[-1]

    points.reverse()
    # Build lower half of the hull.
    lower = [points[0], points[1]]
    for p in points[2:]:
	lower.append(p)
	while len(lower)>2 and not _isRightTurn(lower[-3:]):
	    del lower[-2]
    while len(lower)>2 and not _isRightTurn(lower[-2:]+lower[:1]):
    	del lower[-1]
    us = set(upper)
    for q in lower:
    	if q not in us:
		upper.append(q)

    return tuple(upper)


######################################################################
# Test
######################################################################

def test():
    a = 200
    p = _makeRandomData(300, a, 0)
    c = convexHull(p)
    saveAsEps(p, c, a, "foo.eps")

# test()

######################################################################

if __name__ == '__main__':
    try:
        numPoints = int(sys.argv[1])
        squareLength = int(sys.argv[2])
        path = sys.argv[3]
    except IndexError:
        numPoints = 30
        squareLength = 200
        path = "sample.eps"

    p = _makeRandomData(numPoints, squareLength, addCornerPoints=0)
    c = convexHull(p)
    saveAsEps(p, c, squareLength, path)





from gmisclib import Num
from gmisclib import mcmc
from gmisclib import die
from gmisclib import mcmc_helper
import gmisclib.multivariance_classes as MC
import gmisclib.multivariance_q as MQ
import gmisclib.multivariance_mm as MM



ERGCOVER = 3.0
HUGE = 1e30
				# -1 for a flat prior in inverse sigma,
				# 0 for somewhere in between





def _logp(prms, (x, modelchoice)):
	lp = 0.0
	p = modelchoice.unpack(prms)

	try:
		for x1 in x:
			lp += p.logp(x1)
	except MC.QuadraticNotNormalizable:
		return -HUGE

	return lp





def meanvar(dataset, N, modelchoice=MQ.quadratic):
	"""Given a dataset, this produces a bunch of MCMC estimates
	of plausible means and inverse covariance matrices for the dataset.
	Modelchoice is a particular size model, e.g. an instance of quadratic.
	The output is in the form [ (mean, inv_covar), ... ]."""
	assert N >= 0
	assert len(dataset)>0
	# print 'N=', N
	particular_model = modelchoice(dataset=dataset)
	startpack, varpack = particular_model.start(dataset)
	# print 'SPACK:', startpack
	# print 'VPACK:', varpack
	x = mcmc.bootstepper(_logp, startpack, varpack,
				c=(dataset,particular_model))
	# print 'X!'

	n = particular_model.ndim()
	# print 'ndim=', n
	mcmch = mcmc_helper.stepper(x)
	mcmch.run_to_bottom()
	# print 'bottom', N
	o = []
	for i in range(N):
		mcmch.run_to_ergodic(ERGCOVER/float(N))
		o.append( particular_model.unpack(x.prms()) )
	return o


def test_quadratic():
	d = []
	for l in sys.stdin:
		if l.startswith('#'):
			continue
		if l.strip() == '':
			continue
		a = Num.array( [float(s) for s in l.split()], Num.Float)
		d.append(a)
	for x in meanvar(d, 200, modelchoice=MQ.quadratic):
		print 'Mu:', x.mu
		print 'Inverse:', x.invsigma
		print


def test_diag_quadratic():
	d = []
	for l in sys.stdin:
		if l.startswith('#'):
			continue
		if l.strip() == '':
			continue
		a = Num.array( [float(s) for s in l.split()], Num.Float)
		d.append(a)
	for x in meanvar(d, 200, modelchoice=MQ.diag_quadratic):
		print 'Mu:', x.mu
		print 'Inverse:', x.invsigma
		print

def test_multimu():
	d = []
	for l in sys.stdin:
		if l.startswith('#'):
			continue
		if l.strip() == '':
			continue
		a = l.split()
		af = Num.array([float(s) for s in a[1:]], Num.Float)
		d.append(MM.datum_c(vector=af, classid=a[0]))
	for x in meanvar(d, 200, modelchoice=MM.multi_mu):
		print 'Inverse:', x.invsigma
		print 'Mu:', x.mu
		print


def test_multimu_diag():
	d = []
	for l in sys.stdin:
		if l.startswith('#'):
			continue
		if l.strip() == '':
			continue
		a = l.split()
		af = Num.array([float(s) for s in a[1:]], Num.Float)
		d.append(MM.datum_c(vector=af, classid=a[0]))
	for x in meanvar(d, 200, modelchoice=MM.multi_mu_diag):
		print 'Inverse:', x.invsigma
		print 'Mu:', x.mu
		print


FromName = {'quadratic' : MQ.quadratic, 'diag_quadratic': MQ.diag_quadratic,
		'multi_mu': MM.multi_mu, 'multi_mu_diag': MM.multi_mu_diag
	}

if __name__ == '__main__':
	import sys

	try:
		import psyco
		psyco.full(memory=10000)
	except ImportError:
		pass

	if len(sys.argv) <= 1:
		print 'Usage: multivariance model_name ...'
		print __doc__
		die.exit(0)
	if sys.argv[1] == 'quadratic':
		sys.argv = [sys.argv[0]] + sys.argv[2:]
		test_quadratic()
	elif sys.argv[1] == 'diag_quadratic':
		sys.argv = [sys.argv[0]] + sys.argv[2:]
		test_diag_quadratic()
	elif sys.argv[1] == 'multi_mu':
		sys.argv = [sys.argv[0]] + sys.argv[2:]
		test_multimu()
	elif sys.argv[1] == 'multi_mu_diag':
		sys.argv = [sys.argv[0]] + sys.argv[2:]
		test_multimu_diag()




import sets
import math
import operator as OP

"""This module lets you write mathematical expressions and
build a parse tree.   You can include variables and floats in
the expressions, along with elements of arrays.

You can then walk the parse tree and find out which variables
are used, or which elements of a specified array.
"""

class Array(object):
	def __init__(self, name):
		"""This creates an array.   Arrays are basic
		elements of expressions."""
		self.name = name
	
	def __getitem__(self, key):
		return elementof(self, key)

	def __str__(self, env={}):
		if self.name in env:
			return env[self.name].__str__(env)
		return self.name

	__repr__ = __str__

	def indices(self, env={}):
		raise RuntimeError, 'It is an array'

	def eval(self, env={}):
		raise RuntimeError, 'It is an array'


	def variables(self, env={}):
		if self.name in env:
			return env[self.name].variables(env)
		return sets.Set( ( self.name,) )
	
	def debug(self):
		return '<Array %s>' % self.name

	def _complexity(self):
		return 2


class output_mixin(object):
	def __init__(self):
		pass

	def eval(self, env={}):
		"""This returns an Expression.
		"""
		raise RuntimeError, 'Virtual!'

	def variables(self, env={}):
		"""Any definitions in env are used to resolve otherwise
		undefined names.    This returns a Set containing
		all undefined variable names.
		"""
		raise RuntimeError, 'Virtual!'

	def indices(self, variable, env={}):
		raise RuntimeError, 'Virtual!'

	def debug(self):
		return '<Output_mixin: virtual!>'

	def __str__(self, env={}):
		"""Env is optional. Anything defined in env is used
		to resolve references, but the outcome may still contain
		undefined variables, which are printed as names.
		"""
		raise RuntimeError, 'Virtual!'

	def _complexity(self):
		raise RuntimeError, "Virtual!"


def _is_opNg1(x, op):
	assert op in _operator.Priority
	return isinstance(x, operatorNg1) and x.op==op

def _is_op1(x, op):
	assert op in _operator.Priority
	return isinstance(x, operator1) and x.op==op

def _IMO_float(a, b, indent):
	if a is b:
		return Float(1.0)
	try:
		fb = float(b)
	except TypeError:
		pass
	else:
		# b is a Float
		if fb == 0.0:
			return None
		# b is a nonzero float
		try:
			fa = float(a)
		except TypeError:
			# a is an expression
			# This is the degenerate case expression/float, which is useless
			# and leads to infinite loops.
			return None
		else:
			# Both a and b are floats.
			return Float(fa/fb)
	return None


def _IMO_Name(a, b, indent):
	if isinstance(a, Name) and isinstance(b, Name) and a.name==b.name:
		return Float(1.0)
	return None

def _IMO_elementof(a, b, indent):
	if isinstance(a, elementof) and isinstance(b, elementof):
		# Need to think about the following test a bit...
		# print indent+'is_mul_of element element', a.array, a.index, b.array, b.index
		if str(a.array)==str(b.array) and str(a.index)==str(b.index):
			return Float(1.0)
	return None


def _IMO_products(a, b, indent):
	if _is_opNg1(a, '*'):
		atmp = list(a.operands)
	else:
		atmp = [ a ]
	if _is_opNg1(b, '*'):
		btmp = list(b.operands)
	else:
		btmp = [ b ]
	# print 'atmp=', atmp
	# print 'btmp=', btmp
	if len(atmp) + len(btmp) > 2:
		rtmp = []
		while atmp:
			ai = atmp.pop()
			t = None
			for j in range(len(btmp)):
				# print indent+'Checking is_mul_of', ai, btmp[j]
				t = is_mul_of(ai, btmp[j], indent+'#')
				if t is not None:
					tmp = coerce_into(t)
					if tmp._complexity() < ai._complexity()+btmp[j]._complexity():
						break
			if t is not None:
				# print 'Cancellation', j, ai, btmp[j], '->', t
				atmp.append(t)
				btmp.pop(j)
			else:
				rtmp.append( ai )
		if len(btmp) == 0:
			rv = rtmp[0]
			for rt in rtmp[1:]:
				rv *= rt
			return rv
		# print indent+'IMO: leftovers:', rtmp, atmp, btmp
	return None


def _IMO_neg(a, b, indent):
	if _is_op1(a, 'U-') and _is_op1(b, 'U-'):
		return is_mul_of(a.operand, b.operand, indent+'#')
	if _is_op1(a, 'U-'):
		t = is_mul_of(a.operand, b, indent+'#')
		if t is not None:
			return -t
	if _is_op1(b, 'U-'):
		t = is_mul_of(a, b.operand, indent+'#')
		if t is not None:
			return -t
	return None


def _IMO_exp(a, b, indent):
	if _is_op1(a, 'exp') and _is_op1(b, 'exp'):
		return operator1('exp', a.operand-b.operand)
	return None


def _IMO_pow(a, b, indent):
	if _is_opNg1(a, '**') and len(a.operands)==2:
		if a.operands[1] >= 1.0:
			t = is_mul_of(a.operands[0], b, indent+'#')
			# (tx)^n / x = t^n x^(n-1)
			if t is not None:
				return operatorN('*', operatorN('**', [ t, a.operands[1] ] ),
							operatorN('**', [ b, operatorN('-', a.operands[1], 1) ] )
						)
	if _is_opNg1(a, '**') and len(a.operands)==2 and _is_opNg1(b, '**') and len(b.operands)==2:
		if a.operands[1] >= b.operands[1]:
			t = is_mul_of(a.operands[0], b.operands[0], indent+'#')
			# (tx)^n / x^m = t^n x^(n-m)
			if t is not None:
				return operatorN('*', operatorN('**', t, a.operands[1]),
							operatorN('**', b,
									operatorN('-', [ a.operands[1],
											b.operands[1] ]
										)
								)
						)
	return None


def is_mul_of(a, b, indent=''):
	"""Returns the ratio a/b if a contains b as a factor,
	returns None otherwise.
	The routine does not promise to find all possible factors.
	"""
	# print indent+'PTN.is_mul_of', a, b
	for test in [
			# _IMO_float,
			_IMO_Name, _IMO_elementof,
			_IMO_neg,
			_IMO_products,
			_IMO_exp, _IMO_pow
			]:
		tmp = test(a, b, indent)
		if tmp is not None:
			# print "IMO: a=", a, 'b=', b
			# print 'type(tmp)=', type(tmp), 'test=', tmp
			if tmp._complexity() < a._complexity()+b._complexity():
				return tmp
	return None





class abstract_number(object):
	def __init__(self):
		pass

	def __add__(self, other):
		# print '__add__', str(self), str(other)
		try:
			fs = float(self)
		except TypeError:
			fs = None
		try:
			fo = float(other)
		except TypeError:
			fo = None
		if fo is not None and fs is not None:
			return Float(fo + fs)
		if fs == 0.0:
			return other
		if fo == 0.0:
			return self

		other = coerce_into(other)

		if _is_opNg1(self, '+'):
			a = list(self.operands)
		else:
			a = [ self ]
		if _is_opNg1(other, '+'):
			b = list( other.operands )
		else:
			b = [ other ]

		# print '__add__ a=', a
		# print '__add__ b=', b
		ao = []
		while a:
			ai = a.pop()
			# print 'A.pop =', ai
			for j in range(len(b)):
				# print '__add__IMO ab', j, ai, b[j]
				t1 = is_mul_of(ai, b[j])
				if t1 is not None:
					tmp = coerce_into(t1+1.0)*b[j]
					# print 'tmp=', tmp
					if tmp._complexity() < ai._complexity()+b[j]._complexity()+1:
						a.append(tmp)
						# print 'a.append',  tmp
						b.pop(j)
						ai = None
						break
					else:
						# print "\ttoo complex:", tmp, tmp._complexity(), ai._complexity(), b[j]._complexity()
						pass
				# print '__add__IMO ba', j, b[j], ai
				t2 = is_mul_of(b[j], ai)
				if t2 is not None:
					tmp = coerce_into(t2+1.0)*ai
					# print 'tmp=', tmp
					if tmp._complexity() < ai._complexity()+b[j]._complexity()+1:
						a.append(tmp)
						ai = None
						# print 'a.append', tmp
						b.pop(j)
						break
					else:
						# print "\t too complex:", tmp, tmp._complexity(), ai._complexity(), b[j]._complexity()
						pass
			if ai:
				ao.append(ai)
		# print '__add__ ao =', ao
		# print '__add__ btuple=', b
		return operatorN('+', tuple(ao) + tuple(b))

	__radd__ = __add__
	
	def __sub__(self, other):
		try:
			fs = float(self)
		except TypeError:
			fs = None
		try:
			fo = float(other)
		except TypeError:
			fo = None
		if fo is not None and fs is not None:
			return Float(fs - fo)
		if fs == 0.0:
			return operator1('U-', other)
		if fo == 0.0:
			return self
		other = coerce_into(other)
		if _is_op1(other, 'U-'):
			return operatorN('+', (self, other))
		if _is_op1(self, 'U-'):
			return operator1('U-', operatorN('+', (self, other)))
		return operatorN('-', (self, other))

	def __rsub__(self, other):
		return other.__sub__(self)

	def __pow__(self, other):
		try:
			fs = float(self)
		except TypeError:
			fs = None
		try:
			fo = float(other)
		except TypeError:
			fo = None
		if fo is not None and fs is not None:
			return Float(fs ** fo)
		other = coerce_into(other)
		if _is_float(self, 1.0):
			return Float(1.0)
		if _is_float(other, 1.0):
			return self
		if _is_float(other, 0.5):
			return operator1('sqrt', self)
		return operatorN('**', (self, other))

	def __rpow__(self, other):
		return other.__pow__(self)

	def __mul__(self, other):
		# print 'PTN __mul__', str(self), str(other)
		try:
			fs = float(self)
		except TypeError:
			fs = None
		try:
			fo = float(other)
		except TypeError:
			fo = None
		if fo is not None and fs is not None:
			return Float(fo * fs)
		if fs == 1.0:
			return other
		if fo == 1.0:
			return self
		if fs == -1.0:
			return operator1('U-', other)
		if fo == -1.0:
			return operator1('U-', self)

		other = coerce_into(other)

		if _is_op1(self, 'U-') and _is_op1(other, 'U-'):
			self = self.operand
			other = other.operand

		if _is_opNg1(self, '*'):
			if _is_opNg1(other, '*'):
				# print '\tmore', other.operands
				return self.more(other.operands)
			else:
				# print '\tMore', str(other)
				return self.more( (other,) )
		# print 'PTN_mul_out', self, other
		return operatorN('*', (self, other))

	__rmul__ = __mul__

	def __div__(self, other):
		# print '__div__', self, other
		try:
			fs = float(self)
		except TypeError:
			fs = None
		try:
			fo = float(other)
		except TypeError:
			fo = None
		if fo is not None and fs is not None:
			return Float(fs / fo)
		if fo == 1.0:
			return self
		if fo == -1.0:
			return operator1('U-', self)
		if fo is not None and _is_opNg1(self, '*'):
			modified = False
			atmp = list(self.operands)
			for (i, op) in enumerate(atmp):
				try:
					f = float(op)
				except TypeError:
					pass
				else:
					atmp[i] = Float(f/fo)
					modified = True
			if modified:
				rv = atmp[0]
				for rt in atmp[1:]:
					rv *= rt
				return rv

		other = coerce_into(other)

		t = is_mul_of(self, other)
		if t is not None:
			tmp = coerce_into(t)
			if tmp._complexity() < self._complexity + other._complexity():
				return tmp

		if _is_op1(self, 'U-') and _is_op1(other, 'U-'):
			self = self.operand

		return operatorN('/', (self, other))

	def __rdiv__(self, other):
		return other.__div__(self)
	
	__truediv__ = __div__
	__rtruediv__ = __rdiv__

	def __neg__(self):
		try:
			fs = float(self)
		except TypeError:
			pass
		else:
			return Float(-fs)

		if _is_op1(self, 'U-'):
			# -(-x) = x
			return self.operand
		if _is_opNg1(self, '-') and len(self.operands)==2:
			return operatorN('-', (self.operands[1], self.operands[0]))
		return operator1('U-', self)

	def __pos__(self):
		return self
	
	def __abs__(self):
		if _is_op1(self, 'U-'):
			return operator1('abs', self.operand)
		if _is_op1(self, 'exp') or _is_op1(self, 'abs'):
			return self
		return operator1('abs', self)



_Opfcn = {'-': OP.neg, 'abs': abs,
	'sin': math.sin, 'cos': math.cos, 'exp': math.exp,
	'log': math.log
	}
def _do_math(x, name):
	try:
		f = float(x)
	except TypeError:
		# print 'Name=', name
		return operator1(name, x)
	else:
		return _Opfcn[name](f)


def abs(x):
	if _is_op1(x, 'abs'):	# abs(abs(x)) = x
		return x
	_do_math(x, 'abs')

def cos(x):
	if _is_op1(x, 'abs'):	# cos(abs(x)) = cos(x)
		return _do_math(x.operand, 'cos')
	return _do_math(x, 'cos')

def sin(x):
	return _do_math(x, 'sin')

def exp(x):
	return _do_math(x, 'exp')


def log(x):
	if _is_op1(x, 'exp'):	# log(exp(x)) = x
		return x
	return _do_math(x, 'log')


def sqrt(x):
	if _is_opNg1(x, '**') and len(x.operands)==2 and _is_float(x.operands[1], 2.0):
		# sqrt(x**2) = abs(x)
		return operator1('abs', x.operands[0])
	return _do_math(x, 'sqrt')


class Expression(abstract_number,output_mixin):
	def __init__(self, value):
		abstract_number.__init__(self)
		output_mixin.__init__(self)
		self.x = coerce_into(value)
		self._check()
	
	def _check(self):
		assert isinstance(self.x, Float) or isinstance(self.x, abstract_number), 'Bad type=%s' % str(type(self.x))

	def __iadd__(self, other):
		self._check()
		ci = coerce_into(other)
		# print 'PTN __iadd', type(self.x), self.x.debug(), type(ci), ci.debug()
		self.x = self.x + ci
		# print 'PTNa __iadd', type(self.x), self.x.debug()
		self._check()
		return self

	def __isub__(self, other):
		self._check()
		self.x = self.x - coerce_into(other)
		self._check()
		return self

	def __imul__(self, other):
		self._check()
		self.x = self.x * coerce_into(other)
		self._check()
		return self

	def __idiv__(self, other):
		self._check()
		self.x = self.x / coerce_into(other)
		self._check()
		return self

	def eval(self, env={}):
		return self.x.eval(env)

	def variables(self, env={}):
		return self.x.variables(env)
	
	def indices(self, variable, env={}):
		return self.x.indices(variable, env)

	def __str__(self, env={}):
		# print 'self.x=', self.x, self.x.debug()
		return self.x.__str__(env)

	__repr__ = __str__

	def debug(self):
		return 'E%s' % self.x.debug()

	def __float__(self):
		# print 'PTN float(expression)', self.x
		return float(self.x)
	
	def priority(self):
		return self.x.priority()

	def _complexity(self):
		return self.x.complexity()

class Name(abstract_number,output_mixin):
	def __init__(self, name):
		"""This creates a named variable.   Names are
		basic parts of expressions."""
		abstract_number.__init__(self)
		output_mixin.__init__(self)
		self.name = name

	def __str__(self, env={}):
		if self.name in env:
			return str(env[self.name])
		return self.name

	__repr__ = __str__

	def eval(self, env):
		return env.get(self.name, self.name)

	def variables(self, env={}):
		if self.name in env:
			return env[self.name].variables(env)
		else:
			return sets.Set( ( self.name,) )

	def indices(self, variable, env={}):
		if self.name in env:
			return env[self.name].indices(variable, env)
		else:
			return sets.Set()

	def debug(self):
		return '<Name %s>' % self.name

	def _complexity(self):
		return 2


class Float(abstract_number,output_mixin):
	def __init__(self, v):
		self.v = float(v)

	def eval(self, env={}):
		return self.v

	def variables(self, env={}):
		return sets.Set()
	
	def indices(self, variable, env={}):
		return sets.Set()

	def debug(self, env={}):
		return '<Float %g>' % self.v
	
	def __str__(self, env={}):
		return str(self.v)
	
	def __float__(self):
		# print 'Float -> float', self.v
		return self.v

	def _complexity(self):
		return 1


def _is_float(x, v):
	# print 'PTN, _is_float, type', type(x), x, str(x)
	try:
		f = float(x)
	except TypeError:
		return None
	return f == v


class _operator(abstract_number, output_mixin):
	Priority = {'U-': 5, 'abs': 5, 'sin':5, 'cos':5, 'exp':5,
			'sqrt':5, 'log': 5,
			'+': 2, '-': 2, '*': 3, '/': 3,
			'**': 4,
			'[]': 6
			}

	def __init__(self, operator):
		abstract_number.__init__(self)
		output_mixin.__init__(self)
		self.op = operator

	def priority(self):
		return self.Priority[self.op]


def coerce_into(x):
	if isinstance(x, Expression):
		x = x.x

	try:
		f = float(x)
	except TypeError:
		# print 'coerce_type=', type(x)
		assert isinstance(x, abstract_number)
		assert isinstance(x, output_mixin)
		# print 'Type=', str(type(x))
		return x
	else:
		return Float(x)


class operator1(_operator):
	Printable = {'U-': '-', 'abs': 'abs',
			'sin': 'math.sin', 'cos': 'math.cos',
			'exp': 'math.exp', 'log': 'math.log',
			'sqrt': 'math.sqrt'
			}
	def __init__(self, operator, operand):
		# print 'Operator1: operator=', operator
		_operator.__init__(self, operator)
		self.operand = operand
		assert self.op

	def __str__(self, env={}):
		return '%s(%s)' % (self.Printable[self.op], str(self.operand))
	__repr__ = __str__
	
	def eval(self, env={}):
		_do_math( self.op, self.operand.eval(env) )

	def variables(self, env={}):
		return self.operand.variables(env)

	def indices(self, variable, env={}):
		return self.operand.indices(variable, env)

	def debug(self):
		return '<op1:%s %s>' % (self.op, self.operand.debug())

	def _complexity(self):
		return 1 + len(self.operand)


class elementof(_operator):
	def __init__(self, array, index):
		_operator.__init__(self, '[]')
		self.array = array
		self.index = index
	
	def __str__(self, env = {}):
		index = repr(env.get(self.index, self.index))
		array = str(env.get(self.array, self.array))
		return '%s[%s]' % (array, index)
	__repr__ = __str__
	

	def variables(self, env={}):
		tmp = sets.Set()
		tmp.update( self.array.variables(env) )
		try:
			v = self.index.variables(env)
			tmp.update( v )
		except AttributeError:
			pass
		return tmp

	def indices(self, variable, env={}):
		tmp = sets.Set()
		if isinstance(self.array, Array) and self.array.name==variable:
			if isinstance(self.index, abstract_number):
				tmp.add( self.index.eval(env) )
			else:
				tmp.add(self.index)
		return tmp

	def debug(self):
		if isinstance(self.array, Array):
			a = self.array.debug()
		else:
			a = str(self.array)
		if isinstance(self.index, abstract_number):
			i = self.index.debug()
		else:
			i = str(self.index)
		return '%s[%s]' % (a, i)

	# def provably_eq(self, other):
		# return self.array.provably_equal(other.array)


	def _complexity(self):
		if isinstance(self.array, Array):
			a = self.array._complexity()
		else:
			a = 0
		if isinstance(self.index, abstract_number):
			a += self.index._complexity()
		return a



def operatorN(operator, operands):
	if len(operands) == 1:
		return operands[0]
	nf = 0
	for o in operands:
		try:
			float(o)
			nf += 1
		except TypeError:
			pass
	assert nf<len(operands), "All floats!"
	return operatorNg1(operator, operands)


class operatorNg1(_operator):
	def __init__(self, operator, operands):
		_operator.__init__(self, operator)
		assert type(operands) == type( () )
		assert len(operands) > 1
		self.operands = operands
	
	def more(self, items):
		assert type(items) == type( () )
		return operatorNg1(self.op, self.operands + items)
	
	def __str__(self, env={}):
		tmp = []
		p = self.priority()
		# print 'OP=', self.op, p
		# print 'self=', self.debug()
		for o in self.operands:
			# print 'o=', o.debug(), 'OPp=', p
			try:
				if o.priority() <= p:
					# print '() pri=', o.priority()
					tmp.append( '(%s)' % o.__str__(env) )
				else:
					# print 'NO()', type(o), o, o.priority()
					tmp.append( o.__str__(env) )
			except AttributeError, x:
				# print 'ATTRERR', x
				so = str(o)
				if so.startswith('-'):
					tmp.append( '(%s)' % so)
				else:
					tmp.append( so )
		return self.op.join( tmp )
	__repr__ = __str__
	
	def variables(self, env={}):
		tmp = sets.Set()
		for o in self.operands:
			tmp.update( o.variables(env) )
		return tmp
	
	def indices(self, variable, env={}):
		tmp = sets.Set()
		for o in self.operands:
			tmp.update( o.indices(variable, env) )
		return tmp
	
	def debug(self):
		dl = [ x.debug() for x in self.operands ]
		if len(dl) > 5:
			dl = dl[:4] + ['...']
		return '<opNg1:%s %s>' % (self.op, ' '.join(dl))


	def _complexity(self):
		c = 1
		for op in self.operands:
			c += op._complexity()
		return c


if __name__ == '__main__':
	x = 3.0
	z = 3.0
	y = Name('q')
	w = x + y + y
	print 'w=', w
	print 'wz*wz*wz=', (w+z)*(w+z)*(w+z)
	print 'w.variables()=', w.variables()
	print 'w*w.variables()=', (w*w).variables()
	print '---------- arrays --'
	a = Array('p')
	print "a['x']=", a['x']
	print (336.0*a[5])/336.0 - (a[5]*336.0)/336.0
	print a[1]+a[3]+a[1]
	print a[1]
	print a[1]+a[1]
	print (a[1]+a[1]+a[1]+a[1])/4
	print a[1]+y+x
	print (a[1]+y+x)/336
	print (a[1]+a[2]+a[3])/336
	print a[1].variables()
	print a[1].indices('p')
	print (a[1]+a[3]+a[Name('q')]).indices('p')
	print a[1].indices('b')
	print '--------Expressions-----'
	q = Expression(0.0)
	print 'Float(Expression)=', float(q)
	q += w
	print q
	print q.variables()



"""Read and write files in the form
[label]
text
text
text
[label2]
text
text
...

The format is imperfect to the extent that
it does not allow the text to contain things that
look like labels.


When used as a script, it selects regions of such a file
and prints them on the standard output:
Usage: named_block_file.py key1 [key2] <input >output
The sections labelled key1 and key2 will be output in order,
with no separators other than a newline in between.
"""

import g_encode
import re

_label = re.compile(r"""^\[\s*(.+?)\s*\]\s*$""")

_e = g_encode.encoder(notallowed = r']\r\n%')

def _join(lines):
	n = len(lines)
	if n > 0 and lines[n-1] == '\n':
		n -= 1
	return ''.join(lines)


def read(fd):
	inlabel = None
	lines = []
	d = {}
	for l in fd:
		m = _label.match(l)
		if m:
			if inlabel is not None:
				d[inlabel] = _join(lines)
				lines = []
			inlabel = _e.back(m.group(1))
		else:
			lines.append(l)

	if inlabel is not None:
		d[inlabel] = _join(lines)
	return d


def write_key(fd, k):
	fd.writelines('[%s]\n' % _e.fwd(k))

def write_text(fd, t):
	fd.writelines(t)

def write_line(fd, t):
	fd.writelines([t, '\n'])

def write_line_kv(fd, k, v):
	import avio
	fd.writelines([avio.concoct({k: v}), '\n'])

def write(fd, d):
	for (k, v) in d.items():
		write_key(fd, k)
		write_text(fd, v)
		fd.writelines('\n')
	

def test():
	import sys
	write(sys.stdout, read(sys.stdin))

if __name__ == '__main__':
	import sys
	info = read(sys.stdin)
	for k in sys.argv[1:]:
		sys.stdout.writelines(info[k])



#!/usr/bin/env python

import pydoc
import q3html
from q3html import needcopy, swapend
import re
import os
import die
die.debug = 0

_ck_py = re.compile('for|import|def|None')
_name_py = re.compile('.*[.]py$')
_ck_html = re.compile('< *(TITLE|title)|href|img|< *[pP]|< *[aA]')
_name_html = re.compile('.*[.]html$')

def _checkfile(f, repat):
	fd = open(f, "r")
	for l in fd:
		if repat.match(l):
			return 1
	return 0


def _findfiles(dir, namepat, repat):
	die.note("directory", dir)
	die.note("File name regexp", namepat.pattern)
	die.note("File contents regexp", repat.pattern)
	o = []
	l = os.listdir(dir)
	# print "L=", l
	for t in l:
		ts = t.strip()
		x = namepat.match(ts)
		# print "ts=", ts, "match=", x
		if not x:
			continue
		if not _checkfile(ts, repat):
			die.info("File %s doesn't seem to have the correct contents. Ignored." % ts)
			continue
		o.append(ts)
	return o


def find_py(dir):
	return _findfiles(dir, _name_py, _ck_py)


def find_html(dir):
	return _findfiles(dir, _name_html, _ck_html)


if __name__ == '__main__':
	import sys
	sys.path.insert(0, '.')
	target = sys.argv[1]
	if not os.path.isdir(target):
		die.info("Making directory %s" % target)
		os.makedirs(target)
	for n in find_py('.'):
		nh = swapend(n, '.html')
		if not needcopy(n, nh):
			die.dbg("No need to process %s" % n)
			continue
		pydoc.writedoc(swapend(n, ''))
	upload = q3html.easygo()
	htmllist = find_html('.')
	for n in htmllist:
		nt = '%s/%s' % (target, n)
		if not needcopy(n, nt):
			die.dbg("No need to copy %s" % n)
			continue
		die.info("Copying %s -> %s" % (n, nt))
		tmp = os.system("cp %s %s" %( n, nt))
		assert tmp==0



"""This is a helper module to make use of mcmc.py and mcmc_big.py.
It allows you to conveniently run a Monte-Carlo simulation of any
kind until it converges (L{stepper.run_to_bottom}) or until it
has explored a large chunk of parameter space (L{stepper.run_to_ergodic}).

It also helps you with logging the process.

When run in parallel, each processor does its thing more-or-less
independently.   However, every few steps, they exchange notes on
their current progress.   If one finds an improved vertex, it will be
passed on to other processors via MPI.
"""

import sys
import mpi		# This uses pyMPI
import random
from gmisclib import die
from gmisclib import Num
from gmisclib import gpkmisc
from gmisclib import mcmc
from gmisclib import mcmc_helper as MCH
Debug = 0
from gmisclib.mcmc_helper import TooManyLoops, warnevery, test, logger_template


class _counter(object):
	def __init__(self):
		self.count = 0

	def update(self, accepted):
		self.count += accepted

	def get(self):
		return self.count



class _accum_erg(object):
	def __init__(self):
		self.erg = 0.0
		self.last_resetid = -1

	def update(self, x):
		"""Increment an accumulator, but reset it to
		zero if the underlying stepper has called reset().
		The increment is by the ergodicity estimate, so
		C{self.erg} should approximate how many times the
		MCMC process had wandered across the probability
		distribution.
		"""
		tmp = x.reset_id()
		if tmp != self.last_resetid:
			self.erg = 0.0
			self.last_resetid = tmp
		else:
			self.erg += x.ergodic()


	def get(self):
		return self.erg


class comm_offset(object):
	"""This just provides a string of more-or-less random numbers that is identical on
	each processor.  It's used to decide which processor communicates with which.
	"""
	def __init__(self, size):
		self.n = size
		self.i = 1
		self.a = 1
		for p in gpkmisc.primes():
			if p >= size:
				break
			if p>1 and gpkmisc.gcd(p, size)==1:
				self.a = p
				break
		self.m = 197
		for p in gpkmisc.primes():
			if p >= size:
				break
			if p>5 and gpkmisc.gcd(p, size)==1 and gpkmisc.gcd(p, self.a)==1 and gpkmisc.gcd(p*self.a, size)==1:
				self.m = p
				break

	def __call__(self):
		assert self.n > 0
		while True:
			self.i = (self.i*self.m + self.a) % self.n
			if self.i != 0:
				break
		return self.i


class stepper(MCH.stepper):
	def __init__(self, x, maxloops=-1, logger=None):
		die.info('# mpi stepper rank=%d size=%d' % (mpi.rank, mpi.size))
		assert maxloops == -1
		self.maxloops = -1
		if mpi.size > 1:
			self.co = comm_offset(mpi.size)
		else:
			self.co = None
		MCH.stepper.__init__(self, x, maxloops, logger)


	def reset_loops(self, maxloops=-1):
		assert maxloops == -1
		self.maxloops = -1


	def communicate_hook(self):
		self.note('chook iter=%d' % self.iter, 4)
		if mpi.size > 1:
			self.note('chook active iter=%d' % self.iter, 3)
			handle = mpi.irecv(tag=self.MPIID)
			c = self.x.current()
			v = c.vec()
			lp = c.logp()
			co = self.co()
			mpi.send((v, lp, co), (mpi.rank+co)%mpi.size, tag=self.MPIID)
			handle.wait()
			nv, nlp, nco = handle.message
			assert nco == co

			if nlp > lp - self.x.T*random.expovariate(1.0):
				q = self.x._set_current(c.new(nv-v, logp=nlp))
				self.note('communicate succeeded: %s' % q, 1)
			else:
				self.note('communicate not accepted %g vs %g' % (nlp, lp), 1)
				self.x._set_current(self.x.current())
			sys.stdout.flush()


	MPIID = 1241

	# def _not_enough_changes(self, n, ncw):
		# mytmp = n < ncw
		# self.note('_nyc pre')
		# ntrue = mpi.allreduce(int(mytmp), mpi.SUM)
		# self.note('_nyc post')
		# return ntrue*3 >= mpi.size


	def _not_yet_ergodic(self, nc, ncw):
		mytmp = nc.get() < ncw
		self.note('_nye pre', 5)
		ntrue = mpi.allreduce(int(mytmp), mpi.SUM)
		self.note('_nye post', 5)
		return ntrue*2 >= mpi.size


	def _not_at_bottom(self, xchanged, nchg, es, dotchanged, ndot, update_T):
		mytmp = (Num.sometrue(Num.less(xchanged,nchg))
					or es<1.0 or dotchanged<ndot
					or (update_T and self.x.T>1.5)
				)
		self.note('_nab pre', 5)
		ntrue = mpi.allreduce(int(mytmp), mpi.SUM)
		self.note('_nab post', 5)
		return ntrue*4 >= mpi.size


	def join(self, id):
		self.note('pre join %s' % id, 5)
		rootid = mpi.bcast(id)
		assert rootid == id
		self.note('post join %s' % id, 5)

	def note(self, s, lvl):
		if Debug >= lvl:
			sys.stderr.writelines('# %s, rank=%d\n' % (s, mpi.rank))
			sys.stderr.flush()


def precompute_logp(lop):
	"""Does a parallel evaluation of logp for all items in lop.
	"""
	nper = len(lop)//mpi.size
	r = mpi.rank
	mychunk = lop[r*nper:(r+1)*nper]
	for p in mychunk:
		q = p.logp()
		print 'logp=', q, 'for rank', r
	for r in range(mpi.size):
		nc = mpi.bcast(mychunk, r)
		lop[r*nper:(r+1)*nper] = nc
	mpi.barrier()


def is_root():
	return mpi.rank == 0

def size():
	return mpi.size


def test():
	def test_logp(x, c):
		# print '#', x[0], x[1]
		return -(x[0]-x[1]**2)**2 + 0.001*x[1]**2
	x = mcmc.bootstepper(test_logp, Num.array([0.0,2.0]),
				Num.array([[1.0,0],[0,2.0]]))
	print 'rank=', mpi.rank
	thr = stepper(x)
	# nsteps = thr.run_to_bottom(x)
	# print '#nsteps', nsteps
	# assert nsteps < 100
	for i in range(2):
		print 'RTC'
		thr.run_to_change(2)
		print 'RTE'
		thr.run_to_ergodic(1.0)
		print 'DONE'
	thr.close()


if __name__ == '__main__':
	test()



#!/usr/bin/env python

import math
import random
from gmisclib import Num
from gmisclib import mcmc
from gmisclib import mcmc_helper
import gpkavg
from gmisclib import gpkmisc

def P(x):
	assert Num.alltrue(x >= 0)
	sx = Num.sum(x, axis=0)
	return x/sx


def multinomial_logp(x, cF):
	assert len(x.shape) == 1
	c, F = cF
	assert len(c.shape) == 1
	assert x.shape == c.shape
	if Num.sometrue(x <= 0):
		return None
	sx = Num.sum(x, axis=0)
	xe = x / sx
	assert len(xe.shape) == 1
	log_combinations = gpkmisc.log_factorial(Num.sum(c, axis=0))
	for cc in c:
		log_combinations -= gpkmisc.log_factorial(cc)
	normc = x.shape[0] * math.log(sx)**2
	rv = (Num.sum(Num.log(xe)*c, axis=0) + log_combinations - normc) * F
	return rv


def multinomial_fixer(x, c):
	EPS = 1e-8
	while Num.sometrue( x <= 0.0):
		for i in Num.nonzero( x <= 0.0):
			x[i] = -x[i] + EPS
	return x


def information_gained(c, pc=None):
	"""Suppose there is an experiment where you have N conditions
	and in each conditions there are M possible measurements.
	The outcomes are represented by c[N,M] matrix of counts:
	how many times you observe each possible measurement for each
	condition.

	We want to ask what's the information gained by taking
	a measurement.     We assume the probability of
	each condition is a pc[N] vector, which defaults to the
	frequencies in c.  (We assume that the choice of condition is
	not a random variable, but is made in advance, like most
	experiments.)
	"""
	EPS = 1e-6
	F = 1.0

	c = Num.asarray(c, Num.Int)
	N,M = c.shape
	if pc is not None:
		pc = Num.asarray(pc, Num.Float)
	else:
		print 'c=', c
		print 'NS(c)', Num.sum(c+0.5, axis=1)
		pc = Num.sum(c+0.5, axis=1)/Num.sum(c+0.5)
		print 'pc=', pc
		assert pc.shape == (N,)
	assert abs(Num.sum(pc)-1) < EPS

	K = 2*Num.sum(c) + 10
	K2 = int(round(math.sqrt(K)) + 10)
	pstart = (0.5+c) / Num.sum(0.5+c, axis=0)
	tmpP = []
	for i in range(N):
		pV = 0.1*Num.identity(M)/float(M)**1.5
		xp = mcmc.bootstepper(multinomial_logp, pstart[i,:], pV,
					c=(c[i,:],F), fixer=multinomial_fixer)
		s = mcmc_helper.stepper(xp)
		s.run_to_bottom()
		print 'RTB'
		s.run_to_ergodic(5.0)
		print 'RTE'
		tmp = []
		for j in range(K2):
			s.run_to_ergodic(1.0)
			tmp.append( P(xp.prms()) )
		tmpP.append(tmp)
	print 'Q'
	assert len(tmpP) == N
	tmpe = []
	for j in range(K):
		prior = Num.zeros((M,), Num.Float)
		epost = 0.0
		for i in range(N):
			pi = random.choice(tmpP[i])
			assert abs(Num.sum(pi)-1.0) < 0.001
			Num.add(prior, pi*pc[i], prior)
			epost -= pc[i]*Num.sum(Num.log(pi)*pi)
		assert abs(Num.sum(prior)-1.0) < 0.001
		eprior = -Num.sum(Num.log(prior)*prior)
		tmpe.append( eprior - epost )
	print 'R'
	avg, sigma = gpkavg.avg(tmpe, None, 0.0001)
	return (avg, sigma)

if __name__ == '__main__':
	try:
		import psyco
		psyco.full()
	except ImportError:
		pass
	print information_gained(
		[[0,0,10],[0,0,10],[0,0,10]]
		)



#!/usr/bin/env python

import g_exec
import re
import Num


def _rm_bkt(s):
	s = s.strip()
	if s.startswith('[') and s.endswith(']'):
		return s[1:-1]
	return s

def _sq(s):
	return ''.join( [ q.capitalize() for q in s.split() ] )
	

def read_header(f):
	pass1 = re.compile(r'[(]([0-9a-fA-F]+),([0-9a-fA-F]+)[)]\s+[A-Z]+.[0-9].\s+([a-zA-Z0-9_]+):\s*(.*)\s*[(]\s*[0-9]+ bytes\s*[)]$')
	pass2gel = re.compile(r'([a-zA-Z0-9_]+)\s+[(][0-9]+[)]:\s*([x0-9a-fA-F]+)')
	pass2e = re.compile(r'(.*)\s*:\s*(.*)')
	gnpat = re.compile(r'(.*)\s+-\s+Group\s+([x0-9a-fA-F]+)')
	defer = {}
	gnames = {}
	caught = set()
	ipass = 0
	lip = 0
	p2g = None
	p2e = None
	gname = None
	for s in g_exec.getiter_raw('medcon', ['medcon', '-f', f]):
		s = s.rstrip()

		# print '#', s
		if s.startswith('Pass #1: through DICOM reader'):
			ipass = 1
			continue
		elif s.startswith('Pass #2: through Acr/Nema reader'):
			ipass = 2
			continue
		elif gnpat.match(s):
			m = gnpat.match(s)
			gname = m.group(1)
			gnames[ int(m.group(2),16) ] = gname
			continue

		if ipass == 1:
			m = pass1.match(s)
			if not m:
				continue
			k1, k2, k, v = m.groups()
			if k != 'Unknown':
				caught.add( (int(k1,16), int(k2,16)) )
				yield (k, _rm_bkt(v))
			else:
				defer[ (int(k1,16),int(k2,16)) ] = _rm_bkt(v)
		elif ipass == 2:
			if s.startswith('----'):
				continue
			m = pass2gel.match(s)
			if m:
				if m.group(1) == 'Group':
					p2g = m.group(2)
				elif m.group(1) == 'Element':
					p2e = m.group(2)
				lip += 1
				continue
			m = pass2e.match(s)
			if not m:
				continue
			if p2g is None or p2e is None:
				continue
			k1 = int(p2g, 16)
			k2 = int(p2e, 16)
			if (k1, k2) in caught:
				continue
			k, v = m.groups()
			if v.strip() == '<not printed>':
				continue
			if k.strip() == 'Unknown Element':
				continue
			if (k1,k2) in defer:
				del defer[ (k1,k2) ]
			caught.add( (k1, k2) )
			p2g = None
			p2e = None
			yield (_sq(gname) + '.' + _sq(k), v)
	for ((k1,k2),v) in defer.items():
		if k1 in gnames:
			yield ('%s.%#x' % (_sq(gnames[k1]),k2), v)
		else:
			yield ('CODE%#x.%#x' % (k1,k2), v)


class img_with_mx(object):
	"""This is a way of reading in a 2-D image when you don't
	know the final size.   It keeps a larger image around
	and keeps track of the area that has been set.
	"""

	def __init__(self, i, j):
		"""Create an image that contains pixel (i,j)."""
		assert i>=0 and j>=0
		self.img = Num.zeros((2*i+1,2*j+1), Num.Float)
		self.mi = i
		self.mj = j

	def resize(self, inxt, jnxt):
		tmp = Num.zeros((inxt,jnxt), Num.Float)
		tmp[:self.mi+1, :self.mj+1] = self.img[:self.mi+1, :self.mj+1]
		self.img = tmp

	def set(self, i, j, val):
		"""Set a pixel, expanding the allocated space as needed."""
		assert i>=0 and j>=0

		try:
			self.img[i,j] = val
		except IndexError:
			if i >= self.img.shape[0]:
				inxt = 2*i+2
			else:
				inxt = max(i, self.mi) + 1
	
			if j >= self.img.shape[1]:
				jnxt = 2*j+2
			else:
				jnxt = max(j, self.mj) + 1
			self.img.resize(inxt, jnxt)
			self.img[i,j] = val

		if i > self.mi:
			self.mi = i
		if j > self.mj:
			self.mj = j

	def get(self):
		"""Get the image, trimming it to show
		the area that has been set.
		"""
		return self.img[self.mi:0:-1, :self.mj]


def _set(imgs, iimg, i, j, val):
	assert i>=0 and j>=0
	try:
		img = imgs[iimg]
	except KeyError:
		img = img_with_mx(i, j)
		imgs[iimg] = img
	img.set(i, j, val)


def read_body(f):
	pix = re.compile('\s*P[(]\s*([0-9]+),\s*([0-9]+)\s*[)]')
	imgs = {}
	for s in g_exec.getiter_raw('medcon', ['medcon', '-pa', '-f', f]):
		if not s.startswith('#'):
			# print 'Not hash', s
			continue
		a = s.split(':')
		if len(a) != 8:
			# print 'Not 8:', s
			continue
		assert a[0] == '#' and a[2]=='S' and a[4]=='I'
		m = pix.match(a[6])
		assert m
		i = int(m.group(1)) - 1
		j = int(m.group(2)) - 1
		iimg = int(a[1]) - 1
		sl = float(a[3])
		intercept = float(a[5])
		val = sl*float(a[7]) + intercept
		_set(imgs, iimg, j, i, val)
	o = []
	imgl = imgs.items()
	imgl.sort()
	for (j,img) in imgl:
		o.append(img.get())
	return o


def read_imgs(f):
	import gpkimgclass
	h = dict(read_header(f))
	b = read_body(f)
	return [gpkimgclass.gpk_img(h.copy(), bi) for bi in b]

if __name__ == '__main__':
	import sys
	try:
		import psyco
		psyco.full()
	except ImportError:
		pass
	arglist = sys.argv[1:]
	if arglist[0] == '-o':
		arglist.pop(0)
		o = arglist.pop(0)
		tmp = read_imgs(arglist[0])
		assert len(tmp) == 1
		tmp[0].write(o)
	else:
		for (k,v) in read_header(arglist[0]):
			print '%s = %s' % (k, v)



import getopt

def parse(arglist, shortoptions, longoptions) :
	"""Splits apart an command line argument list, and puts the arguments into a dictionary.
	The arguments are a list to be parsed, a string of short one character options,
	and a list of long gnu-style options.  The function returns a dictionary
	mapping between option names and values (if any).   Names in the dictionary
	have had leading dashes removed. """

	optlist, args = getopt.getopt(arglist, shortoptions, longoptions)
	optdict = {};

	def add_to_dict(x, d=optdict) :
		k, v = x
		while(k[0]=='-') :
			k = k[1:]
		d[k] = v

	map(add_to_dict, optlist)
	return (optdict, args)



"""This takes anticorrelated samples from a sequence of data.
In other words, it tends to choose data that have not been seen
frequently before.   With blueness=1, it never shows a sample
n times until all samples have been shown n-1 times.

(Note that it is possible for an item to appear twice in sucession,
if it appears as the Nth item in a set of N, then appears as the first
item in the second set of N.)
"""

import heapq
import random
import math



class bluedata:
	def __init__(self, data, blueness=2.0):
		assert blueness > 0.0
		self.blue = float(blueness)
		self.q = [ (random.random(), datum) for datum in data ]
		heapq.heapify(self.q)


	def add(self, datum):
		heapq.heappush(self.q, (random.random(), datum) )


	def pickiter(self, n):
		"""Pick n items from the data set."""
		for i in range(n):
			u, d = heapq.heappop(self.q)
			heapq.heappush( self.q, (math.floor(u) + self.blue + random.random(), d) )
			yield d

	def pick(self, n):
		"""Pick n items from the data set."""
		return list(self.pickiter(n))

	def split(self, n):
		"""Split the data set into two parts, of size  n and len(data)-n.
		Successive calls to split will avoid having the same items appear
		in the first part.   For instance, given a bluedata of size 100,
		with ten calls to split(10), each datum will appear in the first
		part of the return tuple exactly once.
		"""
		assert n <= len(self.q), "Split requires 0 <= n <= len."
		o1 = []
		o2 = []
		nq = []
		while len(o1) < n:
			u, d = heapq.heappop(self.q)
			o1.append( d )
			heapq.heappush(nq, (int(u) + self.blue + random.random(), d) )
		for (u, d) in self.q:
			o2.append(d)
			heapq.heappush(nq, (int(u) + random.random(), d) )
		self.q = nq
		return (o1, o2)


	def __iter__(self):
		"""This iterator will produce samples forever."""
		while True:
			u, d = heapq.heappop(self.q)
			heapq.heappush(self.q, (int(u) + self.blue + random.random(), d) )
			yield d


	def __len__(self):
		return len(self.q)


	def reset(self):
		"""Forget prior history of usage.   Choices after this
		call are uncorrelated with choices before this call."""
		self.q = [ (random.random(), d) for (u, d) in self.q ]



def test():
	data = [0, 1, 2, 3, 4, 5, 6]
	x = bluedata(data)
	data.sort()
	o1 = x.pick(3) 
	o2 = x.pick(4)
	o = o1 + o2
	o.sort()
	assert o == data
	o1 = x.pick(2)
	o2 = x.pick(5)
	o = o1 + o2
	o.sort()
	assert o == data
	o1 = x.pick(1)
	o2 = x.pick(6)
	o = o1 + o2
	o.sort()
	assert o == data
	o1 = x.pick(7)
	o2 = x.pick(0)
	o = o1 + o2
	o.sort()
	assert o == data
	o = x.pick(len(data))
	o.sort()
	assert o == data

	assert len(x) == len(data)

	tmp = []
	for (i, q) in enumerate(x):
		tmp.append(q)
		if len(tmp) >= len(data):
			break
	tmp.sort()
	assert tmp == data

if __name__ == '__main__':
	test()




"""Markov-Chain Monte-Carlo algorithms.

Here, we do MCMC when -log(p) is a sum of squares."""


import sys
import math
import random
import Num
import RandomArray
import LinearAlgebra
import g_localfit
import avio
import time
import multivariate_normal
import gpkmisc
import die
import g_implements
from mcmc import _read, _start_is_list_a, def_logger, stepper


# These fudge factors are approximately correct so that,
# for a multivariate Gaussian distribution, you will
# get the same answer with minimize=1 as with minimize=0.
# Minimize=0 -> Markov Chain Monte Carlo.
VARFUDGE = 0.707
VF2 = 1.2
EFUDGE = 1.0/2.0
WORRY = 6
MINIMIZE = 0.0
D = 1

__version__ = "$Revision: 1.62 $"


class CantGetPoint(RuntimeError):
	def __init__(self, s):
		RuntimeError.__init__(self, s)

class DontGoHere(ValueError):
	def __init__(self, s):
		ValueError.__init__(self, s)


class NegativeEigenvalue(ValueError):
	def __init__(self, ev):
		ValueError.__init__(self, str(ev))
		self.ev = ev



class BadlyConditioned(RuntimeError):
	def __init__(self, s):
		RuntimeError.__init__(self, s)



class timer:
	def __init__(self):
		self.tot = 0.0
		self.t0 = None
		self.nc = 1.0
	
	def start(self):
		assert self.t0 is None
		self.t0 = time.time()
	
	def stop(self):
		assert self.t0 is not None
		delta = time.time() - self.t0
		assert delta >= 0.0
		self.tot += delta
		self.t0 = None
		self.nc += 1

	def is_running(self):
		return self.t0 is not None

	def running_total(self):
		if self.t0 is not None:
			t = time.time()
			self.tot += t-self.t0
			self.t0 = t
		return self.tot

	def get(self):
		t = self.running_total()
		nc = self.nc + 0.5*self.is_running()
		return t/nc

	def leak(self, fac=0.8):
		self.tot *= fac
		self.nc *= fac


class problem_definition(object):
	@g_implements.make_optional
	@g_implements.make_varargs
	def __init__(self, resid_fcn, c, fixer=None):
		self.c = c
		self._fixer = fixer
		assert callable(resid_fcn)
		self.rf = resid_fcn

	def resid(self, x):
		return (self.rf)(x, self.c)

	def fixer(self, x):
		"""This allows for symmetries in problems."""
		if self._fixer is not None:
			return (self._fixer)(x, self.c)
		return x


class position(object):
	HUGE = 1e30

	@g_implements.make_optional
	def __init__(self, x, problem_def, resid=None):
		g_implements.check(problem_def, problem_definition)
		self.pd = problem_def
		# print "position:x=", x
		self.x = self.pd.fixer(Num.array(x, Num.Float, copy=True))
		assert len(self.x.shape) == 1
		# print "self.x=", self.x
		# self.shape = self.x.shape
		self._r = resid
		self._ssq = None
		self.tmr = None


	def attach_timer(self, tmr):
		self.tmr = tmr



	def resid(self):
		if self._r is None:
			if self.tmr is not None:
				self.tmr.start()
			try:
				fv = self.pd.resid(self.x)
			except DontGoHere, x:
				print "Don't go here", x
				self._r = x
			else:
				if fv is None:
					print 'FV=none'
					self._r = DontGoHere( 'Function returned "None"' )
				else:
					print 'r=', fv
					self._r = Num.asarray(fv, Num.Float)
			if self.tmr is not None:
				self.tmr.stop()

		if isinstance(self._r, DontGoHere):
			if D:
				print 'position: DontGoHere'
				sys.stdout.flush()
			raise self._r
		return self._r



	def badspot(self):
		try:
			self.resid()
		except DontGoHere:
			return 1
		return 0


	def sumsq(self):
		if self._ssq is None:
			# print 'Computing sumsq' 
			try:
				tmp = self.resid()
			except DontGoHere:
				self._ssq = self.HUGE
			else:
				print 'tmp=', tmp
				self._ssq = Num.sum(tmp**2)
				if self._ssq > self._r.shape[0] * 1000.0:
					ri = Num.argsort(self._r)
					print '#WARN: Huge ssq:', ri[-1], ri[0], self._r.shape[0], self._r[ri[-1]], self._r[ri[0]]
		return self._ssq


	def logp(self):
		return -self.sumsq()/2.0

	def shape(self):
		return self.x.shape[0]

	def new(self, shift):
		"""Returns a new position, shifted by the specified amount."""
		tmp = position(self.x + shift, self.pd)
		tmp.attach_timer( self.tmr )
		return tmp

	def prms(self):
		return self.x

	def __str__(self):
		return '<POS %s -> %s>' % (str(self.x), str(self._logp))


def linear_least_squares_lambda(a, b, lmbda):
	assert len(a.shape) == 2
	assert len(b.shape) in (1, 2)
	ata = Num.matrixmultiply(Num.transpose(a), a)
	# print 'ata=', ata
	i = Num.identity(ata.shape[0])
	atb = Num.matrixmultiply(Num.transpose(a), b)
	atal = ata + i*lmbda
	ev_ata = LinearAlgebra.Heigenvalues(ata)
	ata = None	# Reclaim memory.
	ev_atal = ev_ata + lmbda	# Eigenvalues of atal
	if not Num.alltrue(ev_atal > 0.0):
		raise NegativeEigenvalue, min(ev_atal)
	x = -LinearAlgebra.solve_linear_equations(atal, atb)
	# print 'a=', a
	# print 'x=', x
	# print 'b=', b
	# print 'ax=', Num.matrixmultiply(a, x)
	# print 'ax+b', Num.matrixmultiply(a, x) + b
	# print '(ax+b)**2', (Num.matrixmultiply(a, x) + b)**2
	fiterr = Num.sum( (Num.matrixmultiply(a, x) + b)**2 )
	a = None	# Reclaim memory
	return (x, fiterr, atal, ev_ata)



def test_llsl_1():
	"""Simplest case - one parameter, one observation."""
	b0 = Num.array( (1,) )
	a0 = Num.array( ((1,),) )
	step, fiterr, atal, ev_ata = linear_least_squares_lambda(a0, b0, 0.0)
	# print step
	# print fiterr
	# print atal
	assert fiterr < 1e-6
	assert Num.alltrue(Num.absolute(step+1)<1e-6)

def test_llsl_1o():
	"""Overdetermined."""
	b0 = Num.array( (1,0) )
	a0 = Num.array( ((1,),(1,),) )
	step, fiterr, atal, ev_ata = linear_least_squares_lambda(a0, b0, 0.0)
	# print step
	# print fiterr
	# print atal
	assert abs(fiterr-0.5) < 1e-6
	assert Num.alltrue(Num.absolute(step+0.5)<1e-6)


def test_llsl_2u():
	"""Underdetermined."""
	b0 = Num.array((1,))
	a0 = Num.array(((1,1),))
	e = 0
	try:
		step, fiterr, atal, ev_ata = linear_least_squares_lambda(a0, b0, 0.0)
	except LinearAlgebra.LinAlgError, x:
		if str(x) == 'Singular matrix':
			e = 1
	assert e == 1, 'Should throw exception.'

	step, fiterr, atal, ev_ata = linear_least_squares_lambda(a0, b0, 1.0)
	# print step
	# print fiterr
	# print atal
	assert abs(step[0]-step[1]) < 0.01
	assert Num.sum(Num.ravel(step)**2) < 0.5
	assert fiterr < 0.5 and fiterr>0.01

	step, fiterr, atal, ev_ata = linear_least_squares_lambda(a0, b0, 0.0001)
	# print step
	# print fiterr
	# print atal
	assert abs(step[0]-step[1]) < 0.01
	assert abs(step[0]+step[1]+1) < 0.01
	assert fiterr < 0.001


def test_llsl_2():
	"""Simplest case - two parameters, two observations."""
	b0 = Num.array( (1,-1) )
	a0 = Num.array( ((1,1),(1,-1)) )
	step, fiterr, atal, ev_ata = linear_least_squares_lambda(a0, b0, 0.0)
	# print step
	# print fiterr
	# print atal
	assert fiterr < 1e-6
	assert Num.sum(Num.absolute(step-[0,-1]))<1e-6


	

SINGPASSES = 10

class Lambda_c:
	__doc__ = """This class remembers and increments Lambda,
	for a Levenberg-Marquart-like algorithm.  Initially, when
	we're trying to find the first acceptable step, the adjustments
	to lambda are large.   Then, once a couple of acceptable
	steps have been found, we make the adjustments to lambda
	smaller and also begin to enforce the optimal ratio of
	accepted steps to rejected steps (1:3).
	Later, we keep enforcing the ratio, and make the adjustments
	to lambda still smaller.
	"""

	EARLY_REJECT = 10.0
	EARLY_ACCEPT = 0.1
	ACCEPT = 0.2
	LATE_ACCEPT = 0.6
	AR_RATIO = 0.25	# If everything is going well, we'd like to accept 25% of the steps.
	MIN = 1e-10
	NEG = 5.0
	SING = 2.0
	LIM_F = 10.0

	def __init__(self, lmbda, np):
		"""lmbda is the initial value;
		np is the number of parameters, which is used to control
		whether adjustments are fine (late) or coarse (early).
		"""
		self._lmbda = lmbda
		self.nacc = 0
		self.np = np
		self.fbek = 1.0
		self.vfac = 1.0

	def adjust_BadEstKluge(self, f):
		self.fbek = 1.0/math.hypot(1.0, f)

	def adjust_VfacKluge(self, vfac):
		self.vfac = vfac

	def fa(self):
		# Npf is the result of a complicated calculation.
		# The question is :
		# Assume that you are simulated annealing, and are in the equilibrium
		# distribution for a quadratic np-dimensional potential.
		# Assume you are stepping with a multivariate gaussian step given
		# by that same potential.  What is the probability that the
		# attempted step will bring you closer to the center (lower the E)?
		# It's 50% for a 1-D distribution, but gets smaller for higher D,
		# asymptotically reaching a constant.  The logic is that along
		# a selected direction, you step closer half of the time,
		# but, you're also stepping sideways, and the sideways part of the
		# step can increase the distance.
		tmp = self.vfac * float(self.np-1)/(1.0 + self.np)
		npf = math.exp(-tmp**2)

		return math.sqrt(self.fbek) * npf

	def fr(self):
		return self.fbek

	def lmbda(self):
		"""Return the current value of lambda."""
		return self._lmbda

	def neg(self, min_eigenval):
		self._lmbda = max(self._lmbda*self.NEG, -1.5*min_eigenval)

	def sing(self):
		self._lmbda *= self.SING

	def accept(self):
		if self.nacc < 2:
			f = self.EARLY_ACCEPT ** self.fa()
		elif self.nacc > self.np:
			f = self.LATE_ACCEPT ** self.fa()
		else:
			f = self.ACCEPT ** self.fa()
		self._lmbda = max(f*self._lmbda, self.MIN)
		self.nacc += 1


	def reject(self):
		if self.nacc < 2:
			f = self.EARLY_REJECT ** self.fr()
		elif self.nacc > self.np:
			f = self.LATE_ACCEPT ** (-self.AR_RATIO * self.fr())
		else:
			f = self.ACCEPT ** (-self.AR_RATIO * self.fr())
		self._lmbda *= f


	def low_lim(self, x):
		x /= self.LIM_F
		if self._lmbda < x:
			if D:
				print 'Lambda limited to smallest ev: %g -> %g' % (self._lmbda, x)
			self._lmbda = x


	def high_lim(self, x):
		x *= self.LIM_F
		if self._lmbda > x:
			if D:
				print 'Lambda limited to largest ev: %g -> %g' % (self._lmbda, x)
			self._lmbda = x


def _evec_write(log_fd, a_vec, a_val, note):
	Nlim = 500
	o = ['note=%s' % note,  'eigenvalue=%g' % a_val]
	# XXX Below error checks shouldn't be needed...
	try:
		sq = Num.absolute(a_vec)**2
	except OverflowError:
		die.info("note=" + note)
		die.info("a=" + str(a_vec))
		raise
	idx = Num.argsort(sq)
	total = Num.sum(sq)
	ssq = 0.0
	n = 0
	for i in range(idx.shape[0]-1, -1, -1):
		j = idx[i]
		o.append( 'c%d=%.3f' % (j, a_vec[j]))
		ssq += sq[j]
		n += 1
		if ssq > total*(1-n/float(Nlim)):
			break
	log_fd.write('#Evec: ' + '; '.join(o) + '\n')
	log_fd.flush()



class SQstepper(stepper):
	def __init__(self, pd, lop, v, minimize=0):
		stepper.__init__(self, pd, v)
		assert check_list_of_positions(lop, pd)
		self.archive = lop
		self.np = self.current().shape()
		self.fcntime = timer()
		self.runtime = timer()

		self.lmbda = Lambda_c(1e-6, np)
		self.np = np
		self.archive = archiver(np, v, pd, self.fcntime)

		ZZZ
		for p in positions:
			p.tmr = self.fcntime
			if self.archive.add_initial( p ):
				break

		ZZZ
		for x in locations:
			if self.archive.add_initial( position(x, pd, tmr=self.fcntime) ):
				break
		self.archive.go()
		self.minimize = minimize
		self.logfd = None
		self.a = None
		self.b = None


	def sumsq(self):
		return self.current().sumsq()


	def _set_current(self, newcurrent):
		self.archive.set_current(newcurrent)
		if self.needs_a_reset():
			self.reset()


	def current(self):
		return self.archive.current()

	def last(self):
		return self.archive.last()


	def _step_ab(self, a, b):
		# print 'A', a
		# print 'B', b
		ipass = 0
		while ipass < SINGPASSES:
			print 'PASS', ipass, 'lambda', self.lmbda.lmbda()
			try:
				step, fiterr, atal, ev_ata = linear_least_squares_lambda(a, b, self.lmbda.lmbda())
			except LinearAlgebra.LinAlgError, x:
				if x.startswith('Singular'):
					die.warn('Singular equations in _step_ab(). Lambda=%g' % self.lmbda.lmbda())
					self.lmbda.sing()
				else:
					raise
			except NegativeEigenvalue, x:
				die.warn('Negative eigenvalue in llsql. Lambda=%g, ev=%g'
						% (self.lmbda.lmbda(), x.ev))
				self.lmbda.neg(x.ev)
			else:
				self.lmbda.low_lim( min(ev_ata) )
				self.lmbda.high_lim( max(ev_ata) )
				break
			ipass += 1
		if ipass >= SINGPASSES:
			raise BadlyConditioned, "Can't get positive eigenvalues: pass=%d lambda=%g" % (ipass, self.lmbda.lmbda())
		here = self.current()
		print 'HERE', here.prms()
		print 'STEP', step

		ctr = here.prms() + step
		print 'CTR', ctr
		sfac = self.minimize
		vfac = (VARFUDGE + (VF2-VARFUDGE)*self.minimize)**2
		self.lmbda.adjust_VfacKluge(vfac)
		print 'sfac=', sfac, 'vfac=', vfac
		return self._trysteps(here.prms() + sfac*step, vfac*LinearAlgebra.inverse(atal), here)


	def _trysteps(self, ctr, var, here):
		MAXTRIES = 10
		mvn = multivariate_normal.multivariate_normal(ctr, var)
		i = 0
		print 'fcntime=', self.fcntime.get(), self.runtime.get()
		while i == 0 or (i < MAXTRIES and 2*self.fcntime.get() < self.runtime.get()):
			newx = mvn.sample()
			print 'NEWX', newx
			sys.stdout.flush()
			accepted = self.trypos(newx, here)
			if accepted:
				if i == 0:
					self.lmbda.accept()
				break
			else:
				if i == 0:
					self.lmbda.reject()
			i += 1
		print 'Accepted:', accepted
		sys.stdout.flush()
		return accepted


	def trypos(self, newx, here):
		print 'NEWX', newx
		sys.stdout.flush()
		new = position(newx, here.pd)
		new.attach_timer(self.fcntime)
		print 'NEWLOGP', new.logp(), here.logp()
		if new.logp() > here.logp()-random.expovariate(EFUDGE/(EFUDGE+self.minimize)):
			self._set_current( new )
			self._set_failed( None )
			accepted = 1
		else:
			self._set_failed( new )
			accepted = 0
		self.archive.add( new )
		return accepted


	def step(self, evmisc=None, atmisc=None, showvec=0):
		"""Returns 0 or 1, depending on whether the step was accepted or not.
		"""
		SQstepper.step(self)
		if evmisc is None:
			evmisc = {}
		if atmisc is None:
			atmisc = {}
		self.runtime.start()
		self.b, self.a, not_locally_linear = get_localfit(self.archive)
		self.lmbda.adjust_BadEstKluge(not_locally_linear)
		o = self._step_ab(self.a, self.b)
		self.print_at(misc=atmisc)
		self.print_ev_diag(misc=evmisc, vectors=showvec)
		self.fcntime.leak()
		self.runtime.stop()
		self.runtime.leak()

		# Just save some memory here:
		self.archive.trim_initial()
		return o

	def status(self):
		o = ['E=%g' % self.sumsq(),
			'fcntime=%s' % self.fcntime.get(),
			'runtime=%s' % self.runtime.get(),
			'lambda=%s' % self.lmbda.lmbda()
			]
		return '; '.join(o) + ';'


	def print_ev_diag(self, misc=None, vectors=0):
		if misc is None:
			misc = {}
		if self.logfd is None:
			return
		log_fd = self.logfd
		curv = Num.matrixmultiply(Num.transpose(self.a), self.a)
	
		if vectors:
			eval, evec = LinearAlgebra.Heigenvectors(curv)
		else:
			eval = LinearAlgebra.Heigenvalues(curv)
			evec = None
	
		mx_e = gpkmisc.N_maximum(eval)
		o = misc.copy()
		o['Max_EV'] = '%6g' % mx_e
		o['Min_EV'] = '%6g' % gpkmisc.N_minimum(eval)
		lg_thr = mx_e/10.0
		sm_thr = mx_e/1e4
		o['N_large_ev'] = Num.sum(Num.greater(eval, lg_thr))
		o['large_thr'] = '%6g' % lg_thr
		o['N_small_ev'] = Num.sum(Num.less(eval, sm_thr))
		o['small_thr'] = '%6g' % sm_thr
		o['Median_EV'] = '%6g' % gpkmisc.N_median(eval)
		o['N_sm_lambda'] = Num.sum(Num.less(eval, self.lmbda.lmbda()))
		o['lambda'] = '%6g' % self.lmbda.lmbda()
		log_fd.write('#EV: ' + avio.concoct(o) + '\n')
		if vectors:
			n = eval.shape[0]
			_evec_write(log_fd, evec[0,:], eval[0], "S")
			if n > 2:
				_evec_write(log_fd, evec[1,:], eval[1], "S")
			if n > 1:
				_evec_write(log_fd, evec[-1,:], eval[-1], "L")
			if n > 3:
				_evec_write(log_fd, evec[-2,:], eval[-2], "L")
		log_fd.flush()


	def print_at(self, misc=None):
		"""Prints a file to match the old C++ optimizer output."""
		if misc is None:
			misc = {}
		if self.logfd is None:
			return
		log_fd = self.logfd
		if self.last() is not None:
			log_fd.write("#move:")
			curr = self.current()
			p_step = curr.prms() - self.last().prms()
			for i in range(p_step.shape[0]):
				log_fd.write(" %6g" % p_step[i])
			log_fd.write('\n')
		log_fd.write('# ' + self.status() + avio.concoct(misc) + '\n')
		log_fd.write('#D: ' + self.archive.describe() + '\n')
		currp = self.current().prms()
		for i in range(self.np):
			log_fd.write(" %g" % currp[i])
		log_fd.write('\n')
		log_fd.flush()
	

	def print_rejected(self, r, misc=None):
		if misc is None:
			misc = {}
		if self.logfd is None:
			return
		log_fd = self.logfd
		o = misc.copy()
		o['E'] = r.sumsq()
		o['lambda'] = self.lmbda.lmbda()
		log_fd.write('#R ' + avio.concoct(o) + '\n')
		log_fd.write('#r: ')
		rp = r.prms()
		for i in range(self.np):
			log_fd.write(" %g" % rp[i])
		log_fd.write('\n')
		log_fd.flush()
	


def test_stepper():
	def _tgf_fn(p, c):
		return p*(10.0, 0.3)

	NH = 100
	ND = 2
	c = None
	p = []
	for i in range(NH):
		p.append( RandomArray.multivariate_normal(Num.zeros((ND,)), Num.identity(ND)) )
	s = SQstepper( _tgf_fn, ND, c, Num.identity(ND, Num.Float),
			locations=p,
			minimize=1
			)
	s.logfd = open("/tmp/foo.log", "w")

	sx = 0.0
	sy = 0.0
	sxx = 0.0
	syy = 0.0
	sxy = 0.0
	NS = 3000
	for i in range(NS):
		acc = s.step()
		cur = s.current()
		cx = cur.prms()
		print 'Q', i, cx[0], cx[1], acc, s.lmbda.lmbda()
		sx += cx[0]
		sy += cx[1]
		sxx += cx[0]*cx[0]
		syy += cx[1]*cx[1]
		sxy += cx[0]*cx[1]
	print sx/NS, sy/NS, sxx/NS, syy/NS, sxy/NS
	assert abs(sx) < 10*math.sqrt(NS)
	assert abs(sy) < 10*math.sqrt(NS)





class archiver:
	__doc__ = """
		This class stores history information on the optimization.
		It's main output is from the routine choose(), which selects
		a set of points from the history from which g_localfit can
		compute the Jacobian matrix.
		"""
	SMALL_VOL = 0.1
	SMALL_PRE = 0.5

	def __init__(self, np, v, pd, fcntime):
		self.np = np
		self.ntake = np+2
		self.history = []
		self.initial = []
		assert isinstance(pd, problem_definition)
		self.pd = pd
		self.deriv_pass = None
		self.na_used = None
		self.ni_used = None
		self.nn_used = None
		self.na_unused = None
		self.newscale = 1.0
		self.description = []
		self.mvn = multivariate_normal.multivariate_normal(Num.zeros((np,), Num.Float), v)
		self.fcntime = fcntime
		self._current = None
		self._last = None


	def trim_initial(self, steps):
		efflen = len(self.history) + len(self.initial)/4
		if efflen > 2*self.np:
			ilen = (2*self.np - len(self.history)) * 4
			if ilen > 0  and  ilen < len(self.initial):
				self.initial[: -ilen] = []
			elif ilen <= 0:
				self.initial = []


	def get_new(self):
		if D:
			print "GET_NEW:", self.newscale
		tries = 0
		x = random.choice(self.history[-self.np:])
		while tries < 50:
			step = self.mvn.sample() * self.newscale
			np = position(x.prms() + step, self.pd)
			np.attach_timer(self.fcntime)
			if D:
				print 'GET ANOTHER:', np.prms()

			if not np.badspot():
				break
			if D:
				print 'BAD SPOT'
		if np.badspot():
			die.info("Quite possibly v is set incorrectly.")
			die.warn("Can't get a new point: failed after %d tries" % tries)
			raise CantGetPoint, ''

		# Let's have a little more than normal freedom here...
		# Arrange to have these points scattered a little more widely
		# than the MCMC distribution.
		if np.logp() > x.logp()-random.expovariate(0.3):
			self.newscale = min(1.5*self.newscale, 1000.0)
		else:
			self.newscale = max(0.7*self.newscale, 0.01)

		sys.stdout.flush()
		return np


	def widen(self):
		self.ntake += 1


	def narrow(self):
		if self.ntake > self.np+1:
			self.ntake -= 1


	def choose_guts(self):
		if D: print "CHOOSE_GUTS"
		current = self.current()
		o = []
		nt = 0.0
		self.na_used = 0
		self.nn_used = 0
		self.ni_used = 0
		self.na_unused = 0
		self.description = []

		# Occasionally add one completely new observation.
		# The intent here is to make sure the set of observations
		# doesn't collapse into a subspace.
		try:
			if nt < self.ntake and random.random()<1.0/math.sqrt(self.np):
				anp = self.get_new()
				self.add(anp)
				o.append( (anp.prms()-current.prms(), anp.resid()) )
				self.description.append('N%.1e' % self.newscale)
				self.nn_used += 1
				nt += 1.0
		except CantGetPoint:
			pass

		# Explicitly put in the current position.
		# If/when it appears in the list, it will get dropped as too close.
		o.append( (current.prms()-current.prms(), current.resid()) )
		nt += 1

		# We want the end of the history:
		score1 = Num.arrayrange(len(self.history))
		logp = Num.array( [ x.logp() for x in self.history ] )
		# We want big logp, but small differences amongst the values
		# of logp are unimportant:
		score2 = Num.argsort( logp + RandomArray.chi_square(1, logp.shape))
		# We will randomly push a few points to the end of the list,
		# just to make sure we're not always using the same data.
		score3 = RandomArray.expovariate(2.0/float(len(self.ntake)), logp.shape)
		score = score2 + score1 + score3
		ordering = Num.argsort(score)

		i = len(self.history) - 1
		# Use the recent observations:
		while nt<self.ntake and i >= 0:
			shi = self.history[ordering[i]]

			# The factor is np, instead of t[1].shape[0], because
			# the sample points all sit in a self.np -dimensional
			# subspace.
			dclose = self.SMALL_PRE*self.np*(self.SMALL_VOL/max(self.ntake, len(o)))**(2.0/self.np)
			closest = 1e30
			for t in o:
				# We don't want points that are too similar to points
				# that we have already chosen.   The dependence on
				# ntake**(-2/np) is so that we can't possibly get into
				# a situation where a reasonably good fit
				# is too close to some other
				# reasonably good fit.
				# Mind you, there's no reason to ignore them.  We just
				# make a small increment to the counter, so we will pick
				# up other points, too.
				dist = Num.sum((t[1]-shi.resid())**2)
				if dist < closest:
					closest = dist

			if D and closest<dclose:
				 print 'Close: %g/%g=%g' % (closest, dclose, math.sqrt(closest/dclose))
			if closest > 0.03*dclose:
				o.append( (shi.prms()-current.prms(), shi.resid()) )
				self.na_used += 1
				self.description.append( '%s%d:%d%+-.2g' % ('Aa'[closest<dclose],
									i,
									len(self.history)-score1[ordering[i]],
									logp[ordering[i]]-logp[ordering[0]]) )
				nt += min(1.0, math.sqrt(closest/dclose))
			else:
				self.na_unused += 1
			i -= 1
		return (i, nt, o)


	def choose(self):
		if D:
			print "CHOOSE"
			sys.stdout.flush()
		tries = 0
		while tries < 20*self.np:
			i, nt, o = self.choose_guts()
			if nt >= self.ntake:
				print 'DESC:', ' '.join(self.description)
				sys.stdout.flush()
				# Get rid of unnecessary history:
				xtra = i - 5 - len(self.history)//2
				if xtra > 0:
					self.history[:xtra] = []
				return o
			nadd = 1 + int(round(1.2*(self.ntake*1.2 - nt)))
			if D:
				print "CHOOSE needs %d more" % nadd
				sys.stdout.flush()
			while nadd>=0 and len(self.initial)>0:
				nadd -= self.add( self.initial.pop(-1), 0)
				tries += 1
			while nadd >= 0:
				nadd -= self.add( self.get_new(), -1)
				tries += 1
		if D:
			sys.stdout.flush()
			sys.stderr.flush()
		raise RuntimeError, "Can't manage to choose enough points."


	def set_current(self, setting):
		self._last = self._current
		self._current = setting


	def current(self):
		return self._current


	def last(self):
		return self._last


	def go(self):
		while len(self.initial)>0 and self.initial[-1].badspot():
			self.initial.pop(-1)
		if len(self.initial) == 0:
			raise ValueError, "All the starting positions are bad."
		x = self.initial.pop(-1)
		self.add( x )
		self._current = x


	def add_initial(self, x):
		"""Returns 1 once there is enough data. Otherwise returns 0."""
		# Check for exact dups without calculating residual.
		assert isinstance(x, position)
		for t in self.initial:
			if x.prms() == t.prms():
				return 0

		self.initial.append(x)
		enough = 0
		while len(self.initial) > 24*self.np:
			self.initial.pop(0)
			enough = 1
		return enough


	def add(self, x, location=-1):
		"""Add a point to the history list if it is generally good.
		Location tells you whether to add to the beginning (0) or end (-1)
		of the history list.
		If adding on the end,
		if the new point is very close to a previous point on the list,
		delete the previous point.
		If adding to the beginning, only add if there is no close point
		later in the list.
		It returns the net number of points added.
		"""

		if x.badspot():
			return 0

		dclose = self.SMALL_PRE*self.np*(self.SMALL_VOL/self.np)**(2.0/self.np)
		i = 0
		ndel = 0
		while i < len(self.history):
			t = self.history[i]
			# We don't want duplicate (or near-duplicate) points.
			# The dependence on
			# ntake**(-2/np) is so that we can't possibly get into
			# a situation where reasonably good fit is too close to some other
			# reasonably good fit.
			dist = Num.sum((t.resid()-x.resid())**2)
			if dist < 0.01 * dclose:
				if location == 0:
					if D:
						print 'ADD - dropped: tooclose:', dist, dclose
					return 0
				del self.history[i]
				ndel += 1
			else:
				i += 1

		if location == -1:
			self.history.append(x)
		else:
			self.history.insert(0, x)

		if D:
			print 'ADD (deleted %d)' % ndel
		return 1-ndel



	def describe(self):
		o = {}
		o['na_used'] = self.na_used
		o['na_unused'] = self.na_unused
		o['ni_used'] = self.ni_used
		o['nn_used'] = self.nn_used
		o['deriv_pass'] = self.deriv_pass
		o['newscale'] = '%.1e' % self.newscale
		o['choose'] = ', '.join(self.description)
		return avio.concoct(o)


def get_localfit(selectprms):
	print 'GET_LOCALFIT'
	np = selectprms.np
	MAXPASS = 6 + np/2
	selectprms.narrow()
	selectprms.deriv_pass = 0
	while selectprms.deriv_pass < MAXPASS:
		const, coef, more1, more2, not_locally_linear = get_1_localfit(selectprms)
		if more1 or (0.7*more2)>random.random():
			if random.random()<0.70:
				selectprms.widen()
		else:
			break
		selectprms.deriv_pass += 1
	return (const, coef, not_locally_linear)


def _tgf_fn(p, c):
	return p


def test_get_localfit():
	NH = 100
	ND = 4
	c = None
	selectprms = archiver(ND, Num.identity(ND), problem_definition(_tgf_fn, None), timer())
	for i in range(NH):
		p = RandomArray.multivariate_normal(Num.zeros((ND,)), Num.identity(ND))
		selectprms.add( position(p, _tgf_fn))
	const, coef, nll = get_localfit(selectprms)
	print 'const', const
	print 'coef', coef
	const_adjust = const - Num.matrixmultiply(selectprms.current().prms(), coef)
	print 'const_adj', const_adjust
	assert Num.sum(Num.absolute(const_adjust)) < 1e-6



def sigma(x):
	xr = Num.ravel(x)
	avg = Num.sum(xr)/xr.shape[0]
	return math.sqrt( Num.sum((xr-avg)**2)/(xr.shape[0]-1) )


def get_1_localfit(selectprms):
	SVLIM = 1e-10
	SVLOGLIM = 6
	TINY = 1e-20
	x = selectprms.choose()
	np = selectprms.np
	vbf = g_localfit.err_before_fit(x)
	coef, const, errs, sv, rank = g_localfit.localfit(x, SVLIM)
	print "np=", np, 'rank=', rank
	print 'err_before_fit=', Num.sum(vbf), 'err_after=', Num.sum(errs)
	if Num.sum(vbf)/vbf.shape[0] > 1000.0:
		print '#WARN: VarianceBeforeFit=', vbf
	if Num.sum(errs)/errs.shape[0] > 1000.0:
		print '#WARN: Errs=', errs

	vr = (errs+TINY)/(vbf+TINY)
	print 'maxr=', max(vr), 'minr=', min(vr), 'medr=', gpkmisc.N_median(vr), 'gt0.5=', Num.sum(vr>0.5), 'n=', vr.shape[0]
	nll = (max(vr)>0.9) + (gpkmisc.N_median(vr)>0.5) + (Num.sum(vr)/vr.shape[0])
	nll += (max(vr)>0.7) + (gpkmisc.N_median(vr)>0.3) + (Num.sum(vr>0.5)>math.sqrt(vr.shape[0]))
	nll += (max(vr)>0.5) + (gpkmisc.N_median(vr)>0.2)
	if nll > 1.0:
		die.note('Maximum localfit residual', max(vr))
		die.note('Median localfit residual', gpkmisc.N_median(vr))
		die.note('Num localfit residual>0.5', '%d/%d' % (Num.sum(vr>0.5), vr.shape[0]))
		die.warn("Residuals don't seem to be locally linear.")
	assert np+1 >= rank
	assert len(sv) <= np+1
	assert coef.shape[1] == np
	more1 = np+1 > rank  or  np >= len(x)
	sigsv = sigma(Num.log(Num.array(sv)+TINY))
	if D:
		print "np=", np, 'rank=', rank, 'len(x)=', len(x)
		print 'sigsv=', sigsv, 'err_before_fit=', Num.sum(vbf), 'err_after=', Num.sum(errs)
		print 'get_1_localfit.sv=', sv
	more2 = sigsv > (SVLOGLIM*float(len(x))/float(np)) or 1.2*(np+1)>=len(x)
	if D:	print 'MORE:', more1, more2
	return (const, coef, more1, more2, nll)



def test_llsl():
	test_llsl_1()
	test_llsl_1o()
	test_llsl_2u()
	test_llsl_2()

def test():
	test_llsl()
	test_get_localfit()
	test_stepper()




def diag_variance(start):
	return gpkmisc.make_diag(gpkmisc.vec_variance(start))






def go(argv):
	"""Run an optimization from command line flags."""
	import load_mod
	global MINIMIZE

	if len(argv) <= 1:
		print __doc__
		die.exit(0)
	python = None
	out = None
	NI = None
	arglist = argv[1:]
	while len(arglist) > 0  and arglist[0][0] == '-':
		arg = arglist.pop(0)
		if arg == '-o':
			out = arglist.pop(0)
		elif arg == '-py':	# path/module
			python = arglist.pop(0)
		elif arg == '-NI':
			NI = int(arglist.pop(0))
		elif arg == '-m':
			MINIMIZE = float(arglist.pop(0))
		elif arg == '--':
			break
		else:
			die.die("Unrecognized flag: %s" % arg)
	
	assert python is not None, "Need to use the -py flag."

	mod = load_mod.load_named_module(python)
	print "# mod=", mod

	if out:
		logfd = open(out, "w")
	else:
		logfd = None


	if not hasattr(mod, "start"):
		start = _read(sys.stdin)
	else:
		start = Num.asarray(mod.start(arglist), Num.Float)
		if len(start.shape) == 1:
			start = [start]

	if NI is None:
		NI = mod.NI
	print "# NI=", NI

	if hasattr(mod, "V"):
		V = Num.asarray(mod.V(start), Num.Float)
	elif _start_is_list_a(start) and len(start)>1:
		V = diag_variance(start)
	else:
		V = Num.identity(start[0].shape[0], Num.Float)

	if hasattr(mod, "init"):
		logptr = mod.init(V.shape, NI, mod.c, arglist)
	else:
		logptr = def_logger(V.shape, NI, mod.c, arglist)

	showvec = getattr(mod, 'showvec', 1)

	fixer = getattr(mod, 'fixer', None)

	sys.stdout.flush()
	x = SQstepper(mod.resid, len(start[0]), V, mod.c, locations=start,
			positions=getattr(mod, "positions", []),
			minimize=getattr(mod, "minimize", MINIMIZE)
			)

	sys.stdout.writelines('[P]\n')
	for i in range(NI):
		x.step(showvec=showvec)
		p = x.prms()
		sys.stdout.flush()
		if logfd is not None:
			logfd.writelines('# ' + x.status() + '\n')
			_print(logfd, '%d'%i , p)
			logfd.flush()
		if logptr is not None:
			logptr.add(p, i)
	if logptr is not None:
		logptr.finish(sys.stdout)



if __name__ == '__main__':
	try:
		import psyco
		psyco.full()
	except ImportError:
		pass

	try:
		go(sys.argv)
	except:
		die.catch('Unexpected exception')
		raise





#!/usr/bin/python

"""This program reads a FIAT format data file
and displays some of the information.
You can select some of the lines with the
-select flag, and the display the selected lines
in several different ways.

Flags:
	- -select expr	Where expr is a python expression
		that returns True or False.   All the normal
		builtins are available.   The data in the
		line is presented as a set of local variables,
		so that if the value in the column labelled
		"filename" is "f005.wav", then you could test
		for 'filename=="f005.wav"'.   Note that everything
		is a string, so you may need to convert to int
		or float before some tests: 'int(nspeakers)<5'.
	- -showav	Displays the entire line in a=v; format.
	- -showfiat	Displays the resulting file in FIAT format,
		preserving the header information.
	- -show format
		This displays the file as a
		text file, using evaluating format (which is a
		snippet of python code that should return a string):
		-show "'%s # %s'%(fname,loudness)"
		will display the fname and loudness columns of the
		selected lines, separated by a hash mark.
	- -rav	Reads the input file in a=v; format rather than fiat.

	- -senv  Exec some python code to set a global variable
		that is visible in the -select expression.
		(e.g. "x=3").
	- -penv  Exec some python code to set a global variable
		that is visible in the -show expression.
		(e.g. "x=3").
	- -strap exception value
"""

from gmisclib import die
from gmisclib import fiatio
from gmisclib import avio
from gmisclib import g2_select
from gmisclib import gpk_writer

SHOWAV = 1
SHOWFIAT = 2


def process(f, read_fcn, o, select):
	if f == '-':
		fd = sys.stdin
	else:
		fd = open(f, 'r')

	h1, d1 = read_fcn(fd)
	o.headers(h1)

	ne1 = 0
	ne2 = 0
	npr = 0
	for (i, t) in enumerate(d1):
		try:
			ok = select.eval(t)
		except NameError, x:
			if ne1 == 0:
				die.warn('NameError in selector: %s on item %d: %s' % (str(x), i+1, avio.concoct(t)))
			ne1 += 1
			ok = False
		if not ok:
			continue

		try:
			o.datum(t)
			npr += 1
		except NameError, x:
			if ne2 == 0:
				die.warn('NameError in show: %s on item %d: %s' % (str(x), i+1, avio.concoct(t)))
			ne2 += 1
	if ne1==len(d1):
		die.warn('NameError in select on each of the %d data.' % len(d1))
	if ne2>0 and npr == 0:
		die.warn('NameError in show on each of the %d prints.' % ne2)


def fiat_read(fn):
	h, d, c = fiatio.read(fn)
	return (h, d)


def avio_read(fn):
	d, c = avio.read(fn)
	return ({}, d)


def _find_lod(lod, attribute, value):
	"""Look through a list of dictionaries, as one might get from fiatio.read
	or avio.read, and pick out the dictionary that contains a specified
	attribute=value pair.
	"""
	for d in lod:
		try:
			if d[attribute] == value:
				return d
		except KeyError:
			pass
	return None


class writer(gpk_writer.writer):
	def comment(self, comment):
		"""Add a comment to the data file."""
		pass

	def header(self, k, v):
		pass
	
	def __init__(self, fd, s):
		gpk_writer.writer.__init__(self, fd)
		self.s = g2_select.selector_c(s)


	def datum(self, data_item):
		self.fd.write(str(self.s.eval(data_item)) + '\n')

	def globals(self, code):
		self.s.globals(code)


if __name__ == '__main__':
	import sys
	arglist = sys.argv[1:]
	select = g2_select.selector_c('True')
	select.g['fiatio'] = fiatio
	select.g['avio'] = avio
	select.g['find'] = _find_lod
	o = avio.writer(sys.stdout)
	read_fcn = fiat_read
	deferred_penv = []
	n_senv = 1
	while arglist and arglist[0].startswith('-'):
		arg = arglist.pop(0)
		if arg == '--':
			break
		elif arg == '-select':
			select.set_code(arglist.pop(0))
		elif arg == '-senv':
			tmp = arglist.pop(0)
			try:
				select.globals(tmp)
			except:
				die.warn('Exception while executing -senv flag number %d: "%s"' %
						(n_senv, g2_select._compact(tmp)))
				raise
			n_senv += 1
		elif arg == '-strap':
			select.set_trap(eval(arglist[0], select.g), eval(arglist[1], select.g))
			arglist = arglist[2:]
		elif arg == '-penv':
			deferred_penv.append( arglist.pop(0) )
		elif arg == '-show':
			o = writer(sys.stdout, arglist.pop(0))
		elif arg == '-showav':
			o = avio.writer(sys.stdout)
		elif arg == '-showfiat':
			o = fiatio.writer(sys.stdout)
		elif arg == '-rav':
			read_fcn = avio_read
		elif arg == '-rfiat':
			read_fcn = fiatio_read
		else:
			die.die('Bad arg: %s' % arg)
	if isinstance(o, writer):
		n_penv = 1
		for tmp in deferred_penv:
			try:
				o.globals( tmp )
			except:
				die.warn('Exception while executing -penv flag number %d: "%s"' %
						(n_penv, g2_select._compact(tmp)))
				raise
			n_penv += 1
	elif deferred_penv:
		die.die("Flag -penv only makes sense with -show flag")

	if len(arglist) < 1:
		print __doc__
		die.die('Not enough arguments')

	for f in arglist:
		process(f, read_fcn, o, select)




"""Beam search through a graph."""

class node:
	__doc__ = """This is a node of a graph, it includes links to other nodes."""

	def __init__(self, cost=0.0, label=None, terminal=0, comment=None):
		"""Create a node, with a specified cost (used in the beam search),
		and a label (arbitrary information).
		Terminal nodes are marked, and terminate the search."""

		self.terminal = terminal
		self.out = []
		self.cost = cost
		self.label = label
		self.comment = comment

	def add(self, nextnode, cost=0.0, label=None):
		"""Add a link from self to nextnode.
		Links can have a cost and label, too."""
		self.out.append( (nextnode, cost, label) )
		return nextnode


	def __str__(self):
		return "<node term=%d label=%s nout=%d cost=%f>" % (self.terminal, str(self.label),
									len(self.out), self.cost)

	__repr__ = __str__
	



class _l_list:
	__doc__ = """Linked list."""

	def __init__(self, contents, link):
		self.link = link
		self.contents = contents

	def walk(self):
		o = [self.contents]
		while self.link is not None:
			self = self.link
			o.append(self.contents)
		return o

	def rwalk(self):
		o = self.walk()
		o.reverse()
		return o



def go(graph, nbeam, cbeam):
	"""Search the graph for the lowest cost routes.
	Returns [ (cost, route), ...] where cost is the cost of a route,
	and route is [node, node, node, ...] the list of nodes on the route.
	Routes are sorted in order of increasing cost.
	"""

	assert isinstance(graph, node), "Non-node starts graph: %s" % str(graph)
	# current = [ (0.0, graph, [graph]) ]
	current = [ (0.0, graph, _l_list(graph, None)) ]
	terminate = [ ]

	while 1:
		nextstep = []
		for (cost, anode, path) in current:
			assert isinstance(anode, node), "Non-node in graph: %s" % str(anode)
			for (nnode, c, ll) in anode.out:
				# print anode, "->", nnode
				newcost = cost + nnode.cost + c
				newpath = _l_list(nnode, path)
				# newpath = path + [nnode]
				if nnode.terminal:
					terminate.append( ( newcost, newpath) )
				else:
					nextstep.append( (newcost, nnode, newpath) )
		if len(nextstep) == 0:
			break
		nextstep.sort()
		if len(nextstep) > nbeam:
			nextstep = nextstep[:nbeam]
		costlimit = nextstep[0][0] + cbeam
		i = 0
		while i < len(nextstep):
			if nextstep[i][0] > costlimit:
				break
			i += 1
		current = nextstep[:i]
	terminate.sort()
	if len(terminate) > nbeam:
		terminate = terminate[:nbeam]
	costlimit = terminate[0][0] + cbeam
	o = []
	for (cost, path) in terminate:
		if cost > costlimit:
			break
		o.append( (cost, path.rwalk() ) )
	return o



def test():
	graph = node(0.0, "A")
	graph.add(node(0.0, "B")).add(node(0.1, "C")).add(node(0.2, "D")).add(node(-0.3, terminal=1, label="E"))
	graph.add(node(1.0, "a")).add(node(1, "b")).add(node(2, terminal=1, label="c"))
	graph.add(node(0.0, label="q", terminal=1), 1.0, label="edge")
	print go(graph, 100, 1000.0)


if __name__ == '__main__':
	test()



"""This computes order statistics on data with weights.
"""

from gmisclib import Num


def wp(data, wt, percentiles):
	"""Compute weighted percentiles.
	If the weights are equal, this is the same as normal percentiles.
	Elements of the C{data} and C{wt} arrays correspond to
	each other and must have equal length (unless C{wt} is C{None}).

	@param data: The data.
	@type data: A L{numpy.ndarray} array or a C{list} of numbers.
	@param wt: How important is a given piece of data.
	@type wt: C{None} or a L{numpy.ndarray} array or a C{list} of numbers.
		All the weights must be non-negative and the sum must be
		greater than zero.
	@param percentiles: what percentiles to use.  (Not really percentiles,
		as the range is 0-1 rather than 0-100.)
	@type percentiles: a C{list} of numbers between 0 and 1.
	@rtype: [ C{float}, ... ]
	@return: the weighted percentiles of the data.
	"""
	assert Num.alltrue(Num.greater_equal(percentiles, 0.0)), "Percentiles less than zero"
	assert Num.alltrue(Num.less_equal(percentiles, 1.0)), "Percentiles greater than one"
	data = Num.asarray(data)
	if wt is None:
		wt = Num.ones(data.shape, Num.Float)
	else:
		wt = Num.asarray(wt, Num.Float)
		assert wt.shape == data.shape
		assert Num.alltrue(Num.greater_equal(wt, 0.0)), "Not all weights are non-negative."
	assert len(wt.shape) == 1
	assert len(data.shape) == 1
	n = data.shape[0]
	i = Num.argsort(data)
	sd = Num.take(data, i, axis=0)
	sw = Num.take(wt, i, axis=0)
	aw = Num.add.accumulate(sw)
	if not aw[-1] > 0:
		raise ValueError, "Nonpositive weight sum"
	w = (aw-0.5*sw)/aw[-1]
	spots = Num.searchsorted(w, percentiles)
	o = []
	for (s, p) in zip(spots, percentiles):
		if s == 0:
			o.append(sd[0])
		elif s == n:
			o.append(sd[n-1])
		else:
			f1 = (w[s] - p)/(w[s] - w[s-1])
			f2 = (p - w[s-1])/(w[s] - w[s-1])
			assert f1>=0 and f2>=0 and f1<=1 and f2<=1
			assert abs(f1+f2-1.0) < 1e-6
			o.append(sd[s-1]*f1 + sd[s]*f2)
	return o



def wtd_median(data, wt):
	"""The weighted median is the point where half the weight is above
	and half the weight is below.   If the weights are equal, this is the
	same as the median.   Elements of the C{data} and C{wt} arrays correspond to
	each other and must have equal length (unless C{wt} is C{None}).

	@param data: The data.
	@type data: A L{numpy.ndarray} array or a C{list} of numbers.
	@param wt: How important is a given piece of data.
	@type wt: C{None} or a L{numpy.ndarray} array or a C{list} of numbers.
		All the weights must be non-negative and the sum must be
		greater than zero.
	@rtype: C{float}
	@return: the weighted median of the data.
	"""
	spots = wp(data, wt, [0.5])
	assert len(spots)==1
	return spots[0]


def test():
	assert Num.allclose(wp([1, 2, 3, 4, 5, 6, 7, 8, 9, 10], None,
					[0.0, 1.0, 0.5, 0.51, 0.49, 0.01, 0.99]),
				[1.0, 10.0, 5.5, 5.6, 5.4, 1.0, 10.0], 0.0001)
	assert Num.allclose(wp([0, 1, 2, 3, 4], [0.1, 1.9, 1.9, 0.1, 1],
					[0.0, 1.0, 0.01, 0.02, 0.99]),
					[0.0, 4.0, 0.0, 0.05, 4.0], 0.0001)


def test_median():
	d = [1,1,1,1,2,2,2,2]
	w = [1,1,1,1,2,1,1,1]
	assert 1.0 <= wtd_median(d,w) <= 2.0
	d = [1.0,1,1,1, 2,2,2,2]
	w = [1,1.1,2,1, 2,1,1,1]
	assert 1.0 <= wtd_median(d,w) <= 2.0
	d = [1,1,1,1,2,3,3,3,3.0]
	w = [1,1,1,1,1.0,1,1,1,1]
	assert abs(wtd_median(d,w)-2.0) < 0.001
	d = [1,1,1,1,2,3,3,3,3.0]
	w = [1,1,1.1,1,1.0,1,1,1,1]
	assert 1.0 <= wtd_median(d,w) <= 2.0
	d = [1,1,1,1,2,3,3,3,3.0]
	w = [1.3,1.3,1.3,1.3,1.0,1,1,1,1]
	assert 1.0 <= wtd_median(d,w) <= 2.0
	d = [1,1,1,1,2,3,4,4.0]
	w = [1.,1.,1.,1.,0.0,1,2,2]
	assert abs(wtd_median(d,w)-3.0) < 0.001


if __name__ == '__main__':
	test_median()
	test()



"""This module allows strings to be encoded into a
reduced subset.   It is designed to work for avio.py,
and to do a minimal mapping, so that the resulting
text is human-readable.    I suppose that Quoted-Printable
encoding would work, too...
Normally, the entry points are fwd() and back(),
but you can also make custom encoder classes.

Note that there are some twiddly points in defining
encoders -- the notallowed and allowed arguments
need to be thought through carefully, as they are
passed into the re module as part of a regular
expression.    Certain characters may give surprising
results.
"""

import string;
import re


__version__ = "$Revision: 1.10 $"


class BadFormatError(RuntimeError):
	def __init__(self, *x):
		RuntimeError.__init__(self, *x)








_backdict = {'0':0, '1':1, '2':2, '3':3, '4':4, '5':5,
		'6':6, '7':7, '8':8, '9':9,
		'a':10, 'A':10, 'b':11, 'B':11,
		'c':12, 'C':12, 'd':13, 'D':13,
		'e':14, 'E':14, 'f':15, 'F':15 }


_specials = [ ('mt', ''),
		(' ', '_'),
		('u', '_'),
		('p', '.'),
		('m', ','),
		('s', ';'),
		('z', '='),
		('t', '\t'),
		('Z', '\033'),
		('M', '&'),
		('T', '%'),
		('l', '/'),
		('K', '\\'),
		('k', '\b'),
		('R', '\r'),
		('L', '\n'),
		('q', '"'),
		('Q', '?'),
		('U', "'"),
		('S', ' '),
		('P', '#')
		]


def _expand_bdict(b):
	o = {}
	for (si, ni) in b.items():
		for (sj, nj) in b.items():
			o[si + sj] = chr(16*ni + nj)
	for (k, v) in _specials:
		assert not _backdict.has_key(k), "Special (%s) collides with hex." % k
		assert not o.has_key(k), "Special (%s) collides with hex or special." % k
		o[k] = v
	return o


_bdict = _expand_bdict(_backdict)


_reb = re.compile('%([0-9a-fA-F][0-9a-fA-F]|'
			+ '|'.join([_c[0] for _c in _specials])
			+ ')')




def _fromhex(x):
	"""Expands a %XX code (or the specials above) into a character."""
	# q = x.string[x.start():x.end()]
	q = x.group(1)
	# assert len(q)==3 and q[0]=='%', 'usage: %xx'
	# return chr(_backdict[q[1]]*16 + _backdict[q[2]]);
	return _bdict[q]


def _rm_nl(s):
	if s.endswith('\n'):
		return s[:-1]
	return s



def _expand_fdict():
	o = {}
	for c in range(256):
		o[chr(c)] = '%%%02x' % c
	for (k,v) in _specials:
		o[v] = '%%%s' % k
	return o


_fdict = _expand_fdict()


def _tohex(x):
	"""Converts a single character in a MatchObject to a %xx escape sequence"""
	q = x.string[x.start()]
	assert len(q)==1, 'tohex operates on a single character'
	# return '%%%02x' % ord(q)
	# print "x=", x.string, "q=(%s)" % q
	return _fdict[q]


class encoder:
	def __init__(self, allowed=None, notallowed=None):
		assert not ( allowed is not None and notallowed is not None), "Either allowed or notallowed, not both."
		if notallowed is not None:
			assert '%' in notallowed, "Sorry: cannot allow '%'."
			self.ref = re.compile('(^\s)|([%s])|(\s$)' % notallowed)
		else:
			if allowed is None:
				allowed = string.letters + string.digits + \
						r"""_!@$^&*()+={}[\]\|:'"?/>.<,\ ~`-"""
			assert not '%' in allowed, "Cannot allow '%'."
			self.ref = re.compile('(^\s)|([^%s])|(\s$)' % allowed)


	def back(self, x):
		"""Converts back from a string containing %xx escape sequences to
		an unencoded string.
		"""
		# if x == '%mt':
			# return ''
		try:
			return _reb.sub(_fromhex, x)
		except KeyError, x:
			raise BadFormatError, "illegal escape sequence: %s" % x


	def fwd(self, x):
		"""Escapes a string so it is suitable for a=v; form.
		Nonprinting characters, along with [;#] are converted
		to %xx escapes (hexadecimal).
		Non-strings will be converted to strings with repr(),
		and can be fed back into the python interpreter.  """
		if not isinstance(x, str):
			x = repr(x)
		if x == '':
			return '%mt'
		# print "x=(%s)" % x
		return self.ref.sub(_tohex, x)




def test():
	e = encoder()
	assert e.back(e.fwd('george')) == 'george'
	assert e.back(e.fwd('hello there')) == 'hello there'
	assert e.back('%sfoo') == ';foo'
	assert e.back('%Sfoo%S%P') == ' foo #'
	assert e.back('%Tfoo') == '%foo'
	assert e.back(e.fwd('%hello')) == '%hello'
	assert e.back(e.fwd(' hello there')) == ' hello there'
	assert e.back(e.fwd(' hello there\t')) == ' hello there\t'
	assert e.back(e.fwd(' hello there\t=')) == ' hello there\t='
	assert e.back(e.fwd(' hello there\t=;#')) == ' hello there\t=;#'
	assert e.back(e.fwd(' hello+_there\t=;#')) == ' hello+_there\t=;#'
	assert e.back(e.fwd('hello+_there\t=;#')) == 'hello+_there\t=;#'
	assert e.fwd('hello there') == 'hello there'

	ee = encoder('abcd')
	assert ee.fwd("cab d") == 'cab%Sd'
	assert ee.fwd("e") == '%65'
	assert ee.fwd("aaaa bbbb") == 'aaaa%Sbbbb'

	ee = encoder(notallowed = ']\n\r%')
	assert '\n' not in ee.fwd('hello world\n\r')
	assert ']' not in ee.fwd('hello]% world\n\r')
	assert ee.back(ee.fwd('hello world\n\r'))=='hello world\n\r'



if __name__ == '__main__' :
	test()
	print "OK: passed tests"



#!/usr/bin/env python

"""This module lets you search through label files
to find particular ngrams."""


__version__ = "$Revision: 1.7 $"

# from xwaves_errs import *

import re
import xwaves_mark
import xwaves_lab
from xwaves_errs import *


WildAnyOne = re.compile('.')

def matches(data, pattern):
	if len(data) != len(pattern):
		return 0
	for (d, p) in zip(data, pattern):
		if hasattr(p, 'match'):
			if not p.match(d):
				return 0
		elif d != p:
			return 0
	return 1
	

def find_mark(ngram, fname, datatype=xwaves_mark.PHONE):
	"""ngram = list of labels.
	fname = file name for xwaves_mark datafile.
	datatype = xwaves_mark.PHONE or xwaves_mark.WORD
	
	This function returns all instances
	(even overlapping instances) of the specified
	N-gram in the file.  The return format
	is [ (end_time, label), ... ],
	where the zeroth entry in the list is the
	symbol before the start of the N-gram.
	It's end time is the beginning of the N-gram.

	In the argument list, the N-gram is an array of labels; the labels
	need to match the file's labels exactly.
	"""

	header, data = xwaves_mark.read(fname)
	data = xwaves_mark.mark_to_lab(data, datatype)
	return find(ngram, data)


def find_lab(ngram, fname, loose=0):
	"""ngram = list of labels.
	fname = file name for xwaves_lab datafile.
	
	This function returns all instances
	(even overlapping instances) of the specified
	N-gram in the file.  The return format
	is [ (end_time, label), ... ],
	where the zeroth entry in the list is the
	symbol before the start of the N-gram.
	It's end time is the beginning of the N-gram.

	In the argument list, the N-gram is an array of labels; the labels
	need to match the file's labels exactly.
	"""

	header, data = xwaves_lab.read(fname, loose)
	return find(ngram, data)



def find(ngram, data):
	"""ngram = list of labels.
	data = list of (time, label, ...) as produced by xwaves_lab.py or similar.
	
	This function returns all instances
	(even overlapping instances) of the specified
	N-gram in the file.  The return format
	is [ [ label, ...], ... ] .
	It is a list of n-grams, and each n-gram is a list of entities,
	and each entities is a tuple which marks when it ends,
	what it is specifically, and perhaps other things.

	In the argument list, the N-gram is an array of labels; the labels
	need to match the file's labels exactly.
	"""
	o = []

	ll = [lbl for (t, lbl) in data ]
	N = len(ngram)
	for i in range(1, max(1, len(data)-N+1)):
		if matches(ll[i:i+N], ngram):
			o.append(data[i-1:i+N])
	return o



# -*- Mode: Python; tab-width: 4 -*-

__author__ = "Samual M. Rushing"
# See http://www.nightmare.com/medusa/memory-leaks.html

import sys
import types
import os
import re

def get_refcounts():
    d = {}
    # collect all classes
    for m in sys.modules.values():
        for sym in dir(m):
            o = getattr (m, sym)
            if type(o) is types.ClassType:
                d[o] = sys.getrefcount (o)
    # sort by refcount
    pairs = [ (x[1], x[0]) for x in d.items() ]
    pairs.sort()
    pairs.reverse()
    return pairs


def print_top_N(N=100, prefix='#TOPref'):
	for (n, c) in get_refcounts()[:N]:
		if n > 8:
			print '%s%10d %s.%s' % (prefix, n, c.__module__, c.__name__)


def readvm():
	fd = open("/proc/%d/status" % os.getpid(), "r")
	lines = fd.readlines()
	fd.close()
	nvms = 0
	for line in lines:
		a = line.split()
		if a[0] == 'VmData:':
			assert a[2] == 'kB'
			nvms = int(a[1])
	return nvms

_a1kid = 1
def alloc1k():
	global _a1kid
	tmp = ('%10d' % _a1kid) * 100
	_a1kid += 1
	return tmp

_vmstat = 0
def vm(comment=''):
	global _vmstat
	nvms = 0
	prefix = '#VMSTAT: '
	size = 0
	tmp1 = readvm()
	memstore = []
	while 1:
		memstore.append( alloc1k() )
		size += 1
		tmp2 = readvm()
		if tmp2 > tmp1:
			break
		# print 'len(ms)', len(memstore), tmp2, tmp1
	nvms = tmp1 + 1 - size
	delta = nvms - _vmstat
	_vmstat = nvms
	print '%s%d # %s' % (prefix, delta, comment)

vm('STARTUP')


def lookvmstat(fname):
	def usage_add(u, id, v):
		try:
			u_id = u[id]
			x = (1 + u_id[0], v+u_id[1])

		except KeyError:
			x = (1, v)
		u[id] = x

	fd = open(fname, 'r')
	vml = re.compile('^#VMSTAT:')
	usage = {}
	while 1:
		l = fd.readline()
		if l == '':
			break
		if vml.match(l):
			a = l.strip().split()
			if a[2] != '#':
				continue
			id = ' '.join(a[3:])
			vec = float(a[1])
			usage_add(usage, id, vec)
	fd.close()
	tmp = [ ( -v[1]/float(v[0]), id, v) for (id, v) in usage.items() ]
	tmp.sort()
	for (fom, id, v) in tmp[:20]:
		print id, v


if __name__ == '__main__':
    vm()
    print_top_N()
    vm()
    print_top_N()
    vm()
    tmp = 'adadsadadasd' * 1000
    vm()
    tmp = None
    vm()





"""This class handles error logging.
Duplicate messages are lumped together.
It will remember notes that will be printed
if there is a later error/warning message.
The notes are printed in chronological order.
"""

import sys
import traceback

__version__ = "$Revision: 1.21 $"

debug = True	# Set to False to disable dbg() messages.
nocompress = False

_name = sys.argv[0]
_ni = _name.rfind('/')
if _ni >= 0 and _ni != len(_name)-1:
	_name = _name[_ni+1:]
del _ni

_stderr = sys.stderr
_stdout = sys.stdout


class counter:
	def __init__(self):
		self.reps = 0
		self.last = None
		self.sequence = 0
		self.memory = {}
		self.stderr = _stderr
		self.stdout = _stdout
		assert self.stderr is not None
		assert self.stdout is not None

	def incoming(self, id, name, s):
		t = (id, name, s)
		# print "# t=", t, "last=", self.last
		if nocompress or t != self.last:
			self.stderr.flush()
			self.stdout.flush()
			self.showreps()
			self.dumpmem()
			self.stderr.write('%s: %s: %s\n' % (id, name, s))
			self.stderr.flush()
			self.last = t
		else:
			self.clearmem()
			self.reps += 1
		self.stderr = _stderr
		self.stdout = _stdout
		assert self.stderr is not None
		assert self.stdout is not None

	def showreps(self):
		if self.last is not None and self.reps>1:
			self.stderr.write('%s: last message repeated %d times.\n'
						% (self.last[0], self.reps))
			self.last = None
			self.reps = 1

	def __del__(self):
		self.showreps()

	def clearmem(self):
		self.memory = {}
		self.sequence = 0

	def dumpmem(self):
		tmp = [(sqn, k, val) for (k, (sqn,val)) in self.memory.items()]
		tmp.sort()
		for (sqn,k,val) in tmp:
			self.stderr.write('#NOTE: %s = %s\n' % (str(k), str(val)))
		self.clearmem()

	def memorize(self, key, value):
		self.memory[key] = (self.sequence, value)
		self.sequence += 1

_q = counter()

def die(s):
	"""Output a fatal error message and terminate."""
	e = 'ERR: %s: %s' % (_name, s)
	exit(1, e)


def warn(s):
	"""Output a non-fatal warning."""
	global _q
	_q.incoming('#WARN', _name, s)


def info(s):
	"""Output useful information."""
	global _q
	_q.incoming('#INFO', _name, s)


def catch(extext=None):
	"""Call this inside an except statement.
	It will report the exception and any other information it has."""
	etype, value, tback = sys.exc_info()
	if extext is None:
		extext = "die.catch: exception caught.\n"
	_stderr.flush()
	_stdout.flush()
	_q.showreps()
	_q.dumpmem()
	traceback.print_exception(etype, value, tback)
	etype = None
	value = None
	tback = None
	_stderr.flush()
	_stdout.flush()
	

def catchexit(extext=None, n=1, text=None):
	"""Call this inside an except statement.  It will report
	all information and then exit."""
	catch(extext)
	exit(n, text=text)

def dbg(s):
	"""Output debugging information, if debug is nonzero."""
	if debug:
		global _q
		_q.incoming('#DBG', _name, s)

def exit(n, text=None):
	"""Exit, after dumping accumulated messages."""
	global _q
	_stderr.flush()
	_stdout.flush()
	_q.showreps()
	_q.dumpmem()
	_stderr.flush()
	if text is not None:
		_stdout.write('%s\n' % text)
		_stdout.flush()
	if text is not None:
		_stderr.write('%s\n' % text)
		_stderr.flush()
	sys.exit(n)


def note(key, value):
	"""Memorize a note, which will be output along with the next error/warning/info message."""
	global _q
	_q.memorize(key, value)


def get(key):
	try:
		return _q.memory.get(key, None)[1]
	except KeyError:
		pass
	return None

if __name__ == '__main__':
	debug = 1
	info("You should see a debug message next.")
	dbg("This is the debug message.")
	debug = 0
	note("gleep", "oldest note")
	note("foo", "bar")
	note("foo", "fleep")
	note("foo", "foo")
	note("farf", "newest note")
	info("You should not see a debug message next.")
	dbg("This is the debug message you shouldn't see.")
	warn("This is a warning.")
	warn("This is a warning.")
	warn("This is a warning.")
	info("It should have been repeated three times.")
	die("This is the end.")






"""This acts like a dictionary, but indexed by integers and
stored rather more efficiently, at least in terms of memory.
It is designed for very sparse arrays.
"""

import Num

class ext_block(object):
	def __init__(self, n, vtype=Num.Float):
		self.v = Num.zeros((n,), vtype)
		self.mask = Num.zeros((n,), Num.bool_)

	def set(self, k, v):
		self.v[k] = v
		self.mask[k] = 1


	def get(self, k, IdxErrArg):
		if self.mask[k]:
			return self.v[k]
		else:
			raise IndexError, IdxErrArg


		def unset(self, key):
			self.mask[key] = 0
			return Num.sum(self.mask) == 0


	def contains(self, value):
		return value in self.v.take(indices=self.mask)
		
class extensible_array(object):
	def __init__(self, bsz=300):
		self.x = {}
		self.sz = bsz
		self._len = 0

	def __len__(self):
		return self._len

	def __setitem__(self, k, v):
		"""Doesn't do slice objects."""
		iblock = k//self.sz
		try:
			block = self.x[iblock]
		except KeyError:
			block = ext_block(self.sz)
			self.x[iblock] = block
		self._len = max(self._len, k)
		block.set(k - iblock*self.sz, v)


	def __getitem__(self, k):
		iblock = k//self.sz
		try:
			block = self.x[iblock]
		except KeyError:
			raise IndexError, k
		return self.get(k - iblock*self.sz, k)
	

	def unset(self, key):
		iblock = key//self.sz
		try:
			block = self.x[iblock]
		except KeyError:
			return
		if block.unset(key - iblock*self.sz):
			del self.x[iblock]


	def __contains__(self, item):
		for b in self.x.itervalues():
			if b.contains(item):
				return True
		return False

	def __iter__(self):
		for b in self.x.itervalues():
			nz = Num.nonzero(b.mask)
			v = b.v
			for i in nz:
				yield v[i]



import Num


def nextpow2(i):
	j = 1
	while j <= i:
		j *= 2
	return j


def hilbert(x, cutoff):
	"""This is a Hilbert transform when cutoff=0."""

	xx = Num.asarray(x, Num.Float)
	np2 = nextpow2(x.shape[0])
	x = Num.zeros((np2,), Num.Float)
	x[:xx.shape[0]] = xx
	assert len(x.shape) == 1
	n = x.shape[0]
	xt = Num.FFT.fft(x)
	tmp = Num.arrayrange(n)
	f = Num.minimum(tmp, n-tmp) * (1 - 2*Num.greater(tmp, n-tmp))
	window = 1.0 + Num.tanh(f/cutoff)
	Num.multiply(xt, window, xt)
	return Num.FFT.inverse_fft(xt)



if __name__ == '__main__':
	import gpkimgclass
	x = gpkimgclass.read('/home/gpk/voicing/mimic.1.1.d')
	print "0"
	DT = 0.030
	y = hilbert(x.d[:,0], DT/x.dt())
	print "F", y.shape
	tmp = gpkimgclass.gpk_img({'CDELT2': x.dt(), 'HILBERT_CUTOFF': DT,
					'BITPIX': -32,
					'CRPIX2': 0, 'CRVAL2': 0.0},
		Num.absolute(y))
	tmp.write("foo.d")



"""A multithreaded version of os.popen2().  Note that the argument list
isn't quite the same."""


import subprocess as S




class pfd(object):
	__doc__ = """Pseudo-file descriptor.  Just like a FD, except
		that it waits for the process at the end of the pipe
		to terminate when you close it.
		"""

	def __init__(self, p):
		self.fd = p.stdout
		self.p = p
		self.mode = "r"
		self.closed = False

		self.readline = self.fd.readline
		self.next = self.fd.next
		self.flush = self.fd.flush
		self.__iter__ = self.fd.__iter__
		self.read = self.fd.read
		self.readlines = self.fd.readlines
		self.fileno = self.fd.fileno

	# def read ... copied from file.read
	def read(self):
		return self.fd.read()

	# def readline ... copied from file.readline
	def readline(self):
		return self.fd.readline()

	# def readlines ... copied from file.readlines
	def readlines(self):
		return self.fd.readlines()

	# def next ... copied from file.next
	def next(self):
		return self.fd.next()

	# def __iter__ ... copied from file.__iter__

	def __iter__(self):
		return self.fd.__iter__()

	# def flush ... copied from file.flush
	def flush(self):
		return

	# def fileno ... copied from file.fileno


	def xreadlines(self, sizehint=-1):
		return self.fd.xreadlines(sizehint)


	def close(self):
		if not self.closed:
			self.fd.close()
			self.closed = True
			self.p.wait()
		return self.p.returncode


	def __del__(self):
		self.close()


def popen2(path, args, bufsize=0):
	"""Forks off a process, and returns the processes
	input and output file descriptors.  The latter is really
	a pfd (defined above).

	Path and args are passed directly into os.execvp().
	"""

	p = S.Popen(args, executable=path,
			stdin=S.PIPE, stdout=S.PIPE, close_fds=True)
	return (p.stdin, pfd(p))


def test():
	si, so = popen2("sed", ["sed", "-e", "s/or/er/"])
	si.write("hello, world!\nsecond line\n")
	si.close()
	assert so.readline() == "hello, werld!\n"
	a = so.readlines()
	assert len(a) == 1
	assert a[0] == "second line\n"
	tmp = so.close()
	assert tmp is None, "tmp=%s" % tmp

if __name__ == '__main__':
	test()
	test()



import Num
import math
import cmath
import dpss
import gpkmisc




def complex_median(x):
	if x.typecode() == Num.Float:
		return gpkmisc.N_median(x)

	r1 = gpkmisc.N_median(x.real)
	i1 = gpkmisc.N_median(x.imag)
	phasor = cmath.exp(1j*math.pi/4)
	t = x*phasor
	r2 = gpkmisc.N_median(t.real)
	i2 = gpkmisc.N_median(t.imag)
	return 0.5*(complex(r1,i1) + complex(r2,i2)/phasor)


def one_median(binw, center, v, nbins=None):
	"""Local power in a signal.
	dt and binw are measured in bins.
	"""
	if nbins is None:
		nbins = 3
	assert type(binw) == type(1)
	assert binw>=1
	hbw = binw/2.0

	if center == "?data length":	# How much data do you need?
		return int(math.ceil(hbw*(4*nbins+1))) + binw

	w = 0.5/binw
	# print "binw=", binw, "1/w=", 1/w, "nbins=", nbins, "center=", center
	lmbda, window = dpss.dpsscache(w, 1, binw)
	assert lmbda[0]>0.7  and  lmbda[0]<=1.0
	v = Num.asarray(v)
	t = Num.zeros((4*nbins+1,), v.typecode())
	m = 0
	for k in range(-2*nbins, 2*nbins+1):
		ctr = center + hbw * k
		s = int(round(ctr - hbw))
		if s < 0:
			continue
		e = s + binw
		if e >= len(v):
			continue
		t[m] = Num.sum(v[s:e] * window[0])
		# print "bin", k, "s=", s, "e=", e, "t=", t[m]
		m += 1
	return complex_median(t[:m]) / Num.sum(window[0])



def stepped_median(dt, binw, v, nbins=None):
	"""Local power in a signal.
	dt and binw are measured in bins.
	"""
	binw = int(round(binw))
	v = Num.asarray(v)
	nb = int(round(len(v)/dt))
	o = Num.zeros((nb,), v.typecode())
	for i in range(nb):
		o[i] = one_median(binw, dt*i, v, nbins)
	return o


def pwr(dt, binw, v, nbins=3):
	v = Num.asarray(v, Num.Float)
	return stepped_median(dt, binw, v**2, nbins)


if __name__ == '__main__':
	print pwr(1, 3, [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
	# print pwr(1, 3, [0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
	# print pwr(1, 3, [0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1])



#!/usr/bin/env python
"""
You can control the axis tick and grid properties
"""
import pylab
import sys
import Num

def plot_oneaxis(dlist):
	"""dlist is a list  of (label, (ctr, uerr, derr))."""
	pylab.figure()
	ax = pylab.subplot(111)
	ax.set_yticks( [ pos[0] for (lab,pos) in dlist ] )
	ax.set_yticklabels( [ lab for (lab,pos) in dlist ] )
	ax.set_xticks( [] )
	ax.set_xticklabels( [] )
	regions = []
	y = []
	err = []
	x = []
	for (lab, (ctr, uerr, derr)) in dlist:
		y.append(ctr)
		err.append((derr,uerr))
		xj = 0.5
		for (l,h,xi) in regions:
			if l <= ctr <= h:
				xj += 1
		regions.append((ctr-1.3*derr, ctr+1.3*uerr, xj))
		x.append(xj)
	pylab.errorbar(x=Num.array(x), y=Num.array(y),
				yerr=Num.array(err)
				)





if __name__ == '__main__':
	(d,c,x) = process(sys.stdin)

	plot_oneaxis(dlist)
	pylab.show()
